#!/usr/bin/env python

from Tkinter import *
import tkFileDialog
import tkMessageBox
import time
import thread
import sys, os
from Gibbs_Frame import script, VKMLframe

import StringIO
from keyword import iskeyword
from tokenize import tokenize, TokenError
from tokenize import NAME, NUMBER, STRING, COMMENT, OP, ERRORTOKEN
KEYWORD, DOC = 1001, 1002

PERM_TITLE = 'GibbsLive - '


__version__ = "1.0.0.4"


_bg = "#e6e6e6"           # Default: "#e6e6e6"
_fg = "#000000"           # Default: "#000000"
_bg1 = "#004080"          # Default: "#004080"
_fg1 = "#ffffff"          # Default: "#ffffff"
_bg2 = "#ffffff"          # Default: "#ffffff"
_fg2 = "#404040"          # Default: "#404040"


_iconset = "gtk"          # Default: "gtk"

_wordwrap = "word"        # Default: "word"


if (sys.platform == "win32"):
    fn_standard = "Tahoma 8"
    fn_courier = "Courier\ New 10"
else:
    fn_standard = "Helvetica -12"
    fn_courier = "Courier -12"


###############################################################################
#
# Main window
#
###############################################################################
class Application(Tk):

    def create_widgets(self):
        bg2 = _bg2
        fg2 = _fg2
        fn = fn_standard
        fn2 = fn_courier
        
        
        
        self.default_status_bar_text = ""


        self._menu_frame = Frame(self,
                                 borderwidth=0,
                                 relief="flat")
        self._menu_frame.pack(side="top",expand=1,fill="x")

        self.menu_frame = Frame(self._menu_frame,
                                borderwidth=2,
                                relief="raised")
        self.menu_frame.pack(side="top",expand=1,fill="x")


        self.file_menubutton = Menubutton(self.menu_frame,
                                          font=fn,
                                          relief="flat",
                                          text="File",
                                          underline=None,
                                          direction="below")
        self.file_menubutton.pack(side="left")

        self.edit_menubutton = Menubutton(self.menu_frame,
                                          font=fn,
                                          relief="flat",
                                          text="Edit",
                                          underline=None,
                                          direction="below")
        self.edit_menubutton.pack(side="left")

        self.view_menubutton = Menubutton(self.menu_frame,
                                          font=fn,
                                          relief="flat",
                                          text="View",
                                          underline=None,
                                          direction="below")
        self.view_menubutton.pack(side="left")

        self.insert_menubutton = Menubutton(self.menu_frame,
                                            font=fn,
                                            relief="flat",
                                            text="Insert",
                                            underline=None,
                                            direction="below")
        self.insert_menubutton.pack(side="left")

        self.tools_menubutton = Menubutton(self.menu_frame,
                                           font=fn,
                                           relief="flat",
                                           text="Tools",
                                           underline=None,
                                           direction="below")
        self.tools_menubutton.pack(side="left")

        self.extra_menubuttons = Frame(self.menu_frame,
                                       borderwidth=2,
                                       relief="flat")
        self.extra_menubuttons.pack(side="left")

        self.help_menubutton = Menubutton(self.menu_frame,
                                          font=fn,
                                          relief="flat",
                                          text="Help",
                                          underline=None,
                                          direction="below")
        self.help_menubutton.pack(side="left")

		############################################################
        self.command_frame = VKMLframe(self._menu_frame, self)
        self.command_frame.pack(side="bottom",expand=1,fill="x")
        

		############################################################

        self._status_frame = Frame(self,
                                   borderwidth=0,
                                   relief="flat")
        self._status_frame.pack(side="bottom",expand=1,fill="x")

        self.status_frame = Frame(self._status_frame,
                                  borderwidth=2,
                                  relief="flat")
        self.status_frame.pack(side="top",expand=1,fill="x")

        self.status_bar = Label(self.status_frame,
                                font=fn,
                                relief="flat",
                                text=self.default_status_bar_text,
                                justify="left")
        self.status_bar.pack(side="top",expand=1,fill="x")

        self._dc_frame = Frame(self,
                               borderwidth=0,
                               relief="flat")

        
        
        self.document_frame = Frame(self._dc_frame,
                                    borderwidth=2,
                                    relief="flat")
        self.document_frame.pack(side="top",expand=1,fill="both")

        self.scrollbar = Scrollbar(self.document_frame,
                                   relief="flat")

        self.document = Text(self.document_frame,
                             bg=bg2,
                             fg=fg2,
                             insertbackground=fg2,
                             font=fn2,
                             relief="sunken",
                             width=500,
                             height=25,
                             wrap=_wordwrap)
                             
        
        self.document["yscrollcommand"] = self.scrollbar.set
        self.scrollbar["command"] = self.document.yview

        self.document.focus_set()

        self._filename = ""

        self._filetypes = [
            ("Python","*.py *.pyc"),
            ("All", "*")
            ]

        self._addons = []

        self._clipboard = ""
            
    def create_menu(self):
        fn = fn_standard
        
        self.menubar = Menu(self,
                            font=fn,
                            relief="raised",
                            tearoff=0)

        self.file_menu = Menu(self.file_menubutton,
                              font=fn,
                              relief="raised",
                              tearoff=0)
        self.file_menubutton["menu"] = self.file_menu

        self.file_menu.add_command(label="New",
                                   accelerator="Ctrl-N",
                                   command=self.new_file,
                                   underline=None)

        #self.file_menu.add_command(label="New Window",
        #                           accelerator="Ctrl-Shift-N",
        #                           command=self.newwindow,
        #                           underline=None)

        self.file_menu.add_separator()

        self.file_menu.add_command(label="Open",
                                   accelerator="Ctrl-O",
                                   command=self.open_file,
                                   underline=None)

        self.file_menu.add_separator()

        self.file_menu.add_command(label="Save",
                                   accelerator="Ctrl-S",
                                   command=self.save_file,
                                   underline=None)

        self.file_menu.add_command(label="Save As",
                                   accelerator="F12",
                                   command=self.save_as_file,
                                   underline=None)
        
        self.file_menu.add_separator()

        self.file_menu.add_command(label="Minimize",
                                   accelerator="Ctrl-M",
                                   command=self.minimize,
                                   underline=None)

        self.file_menu.add_separator()

        self.file_menu.add_command(label="Close",
                                   accelerator="Alt-F4",
                                   command=self.closewindow,
                                   underline=None)

        self.file_menu.add_command(label="Exit",
                                   accelerator="Ctrl-Q",
                                   command=self.destruct,
                                   underline=None)
        
        self.menubar.add_cascade(label="File",
                                 menu=self.file_menu,
                                 underline=None)

        self.edit_menu = Menu(self.edit_menubutton,
                              font=fn,
                              relief="raised",
                              tearoff=0)
        self.edit_menubutton["menu"] = self.edit_menu

        self.edit_menu.add_command(label="Cut",
                                   accelerator="Ctrl-X",
                                   command=self.cut_text,
                                   underline=None)

        self.edit_menu.add_command(label="Copy",
                                   accelerator="Ctrl-C",
                                   command=self.copy_text,
                                   underline=None)

        self.edit_menu.add_command(label="Paste",
                                   accelerator="Ctrl-V",
                                   command=self.paste_text,
                                   underline=None)

        self.edit_menu.add_command(label="Delete",
                                   accelerator="",
                                   command=self.delete_text,
                                   underline=None)

        self.edit_menu.add_separator()

        self.edit_menu.add_command(label="Select All",
                                   accelerator="",
                                   command=self.select_all,
                                   underline=None)

        self.menubar.add_cascade(label="Edit",
                                 menu=self.edit_menu,
                                 underline=None)

        self.view_menu = Menu(self.view_menubutton,
                              font=fn,
                              relief="raised",
                              tearoff=0)
        self.view_menubutton["menu"] = self.view_menu

        self.view_menu.add_command(label="Full Mode",
                                   accelerator="",
                                   command=lambda event=None: self.pack_widgets(1,1,1,"right"),
                                   underline=None)

        self.view_menu.add_command(label="Left-Handed Mode",
                                   accelerator="",
                                   command=lambda event=None: self.pack_widgets(1,1,1,"left"),
                                   underline=None)

        self.menubar.add_cascade(label="View",
                                 menu=self.view_menu,
                                 underline=None)

        self.insert_menu = Menu(self.insert_menubutton,
                                font=fn,
                                relief="raised",
                                tearoff=0)
        self.insert_menubutton["menu"] = self.insert_menu

        self.insert_menu.add_command(label="Date & Time",
                                     accelerator="F5",
                                     command=self.date_time,
                                     underline=None)

        self.menubar.add_cascade(label="Insert",
                                 menu=self.insert_menu,
                                 underline=None)

        self.tools_menu = Menu(self.tools_menubutton,
                               font=fn,
                               relief="raised",
                               tearoff=0)
        self.tools_menubutton["menu"] = self.tools_menu

        self.tools_menu.add_command(label="Character Count",
                                    accelerator="Alt-Shift-D",
                                    command=self.charactercount,
                                    underline=None)

        self.tools_menu.add_separator()
        
        self.tools_menu.add_command(label="Run Script:",
                                    accelerator="F5",
                                    command=self.run,
                                    underline=None)

        self.tools_menu.add_command(label="Options",
                                    accelerator="",
                                    command=self.options_window,
                                    underline=None)

        self.menubar.add_cascade(label="Tools",
                                 menu=self.tools_menu,
                                 underline=None)

        self.help_menu = Menu(self.help_menubutton,
                              font=fn,
                              relief="raised",
                              tearoff=0)
                              
        self.help_menubutton["menu"] = self.help_menu

        self.help_menu.add_command(label="Help",
                                   accelerator="F1",
                                   command=self.help_window,
                                   underline=None)

        self.help_menu.add_command(label="Add-ons",
                                   accelerator="",
                                   command=self.show_addons,
                                   underline=None)

        self.help_menu.add_command(label="About",
                                   accelerator="",
                                   command=self.about_box,
                                   underline=None)

        self.menubar.add_cascade(label="Help",
                                 menu=self.help_menu,
                                 underline=None)

        self.menubar.add_separator()

        self.secret_menu = Menu(self,
                                font=fn,
                                relief="raised",
                                tearoff=0)

        self.document.bind("<Control-N>",self.new_file)
        self.document.bind("<Control-n>",self.new_file)

        self.document.bind("<Control-O>",self.open_file)
        self.document.bind("<Control-o>",self.open_file)

        self.document.bind("<Control-S>",self.save_file)
        self.document.bind("<Control-s>",self.save_file)

        self.document.bind("<F12>",self.save_as_file)

        self.document.bind("<Control-M>",self.minimize)
        self.document.bind("<Control-m>",self.minimize)

        self.document.bind("<Alt-F4>",self.closewindow)

        self.document.bind("<Control-Q>",self.destruct)
        self.document.bind("<Control-q>",self.destruct)
        self.document.bind("<Control-Alt-Delete>",self.destruct)

        self.document.bind("<Control-X>",self.cut_text)
        self.document.bind("<Control-x>",self.cut_text)

        self.document.bind("<Control-C>",self.copy_text)
        self.document.bind("<Control-c>",self.copy_text)

        self.document.bind("<Control-V>",self.paste_text)
        self.document.bind("<Control-v>",self.paste_text)

        self.document.bind("<Alt-D>",self.date_time)

        self.document.bind("<F5>", self.run)
        self.document.bind("<F1>",self.help_window)

        self.document.bind("<3>",self.show_right_menu)
        self.document.bind("<Control-1>",self.show_right_menu)

    def run(self, event=None):
        self.save_file()
        if '--nanohub' not in sys.argv:
            os.system("./safety.py " + self._filename + " --gui --quit &")
        else:
            os.system("safety.py " + self._filename + ' --gui --quit &')

    def pack_widgets(self,
                     pack_menu_frame=1,
                     pack_status_frame=1,
                     pack_scrollbar=1,
                     scrollbar_side="right"):
        self._menu_frame.pack_forget()
        self._status_frame.pack_forget()
        self._dc_frame.pack_forget()
        self.scrollbar.pack_forget()
        self.document.pack_forget()
        if (pack_menu_frame == 1):
            self._menu_frame.pack(side="top",expand=0,fill="x")
        if (pack_status_frame == 1):
            self._status_frame.pack(side="bottom",expand=0,fill="y")
        if (pack_scrollbar == 1):
            self.scrollbar.pack(side=scrollbar_side,expand=1,fill="y")
        self._dc_frame.pack(side="top",expand=1,fill="both")
        self.document.pack(side="top",expand=1,fill="y")

    def show_right_menu(self,event):
        self.menubar.tk_popup(event.x_root,event.y_root)

    def show_secret_menu(self,event):
        self.secret_menu.tk_popup(event.x_root,event.y_root)

    def change_toolbar_icons(self,iconset=_iconset):
        try:
            self.command_new["bitmap"] = "@icons/%s/new.xbm" % (iconset)
            self.command_open["bitmap"] = "@icons/%s/open.xbm" % (iconset)
            self.command_save["bitmap"] = "@icons/%s/save.xbm" % (iconset)
            self.command_about["bitmap"] = "@icons/%s/about.xbm" % (iconset)
            self.command_exit["bitmap"] = "@icons/%s/exit.xbm" % (iconset)
        except:
            pass
        try:
            self.command_new.photo = PhotoImage(file="icons/%s/new.gif" % (iconset))
            self.command_open.photo = PhotoImage(file="icons/%s/open.gif" % (iconset))
            self.command_save.photo = PhotoImage(file="icons/%s/save.gif" % (iconset))
            self.command_about.photo = PhotoImage(file="icons/%s/about.gif" % (iconset))
            self.command_exit.photo = PhotoImage(file="icons/%s/exit.gif" % (iconset))
            self.command_new["image"] = self.command_new.photo
            self.command_open["image"] = self.command_open.photo
            self.command_save["image"] = self.command_save.photo
            self.command_about["image"] = self.command_about.photo
            self.command_exit["image"] = self.command_exit.photo
        except:
            pass

    def date_time(self,event=None):
        self.document.insert("insert",time.asctime())

    def cut_text(self,event=None):
        try:
            self._clipboard = self.document.get("sel.first","sel.last")
            self.document.delete("sel.first","sel.last")
        except:
            pass

    def copy_text(self,event=None):
        try:
            self._clipboard = self.document.get("sel.first","sel.last")
        except:
            pass

    def paste_text(self,event=None):
        try:
            self.document.insert("insert",self._clipboard)
        except:
            pass

    def delete_text(self,event=None):
        try:
            self.document.delete("sel.first","sel.last")
        except:
            pass

    def select_all(self,event=None):
        try:
            self.document.tag_add("sel","1.0","end")
        except:
            pass

    def help_window(self,event=None):
        helpbox = HelpBox()
        helpbox.mainloop()

    def about_box(self,event=None):
        aboutbox = AboutBox()
        aboutbox.mainloop()

    def show_addons(self,event=None):
        add_ons_text = "The following PyWord add-ons have been detected..."
        for i in range(len(self._addons)):
            add_ons_text += "\n\n%s - %s" % (self._addons[i][0],self._addons[i][1])
        tkMessageBox.showinfo("Add-ons",add_ons_text)

    def new_file(self, event=None):
        filename = tkFileDialog.asksaveasfilename(filetypes=self._filetypes)
        if filename.endswith('.py'):
            self.command_frame.add_item_to_menu(script(filename,self))
        self.command_frame.set_option(filename.split('.py')[0].split(os.sep)[-1])
        #self.newwindow()
        #self._filename = ""
        #self.document.delete("1.0","end")
        #self.set_title("Untitled")

    def open_file(self,event=None,filename=None):
        if not filename:
            self._filename = tkFileDialog.askopenfilename(filetypes=self._filetypes)
        else:
            self._filename = filename
        if not (self._filename == "" or self._filename == None):
            try:
                f = open(self._filename,"r")
                f2 = f.read()
            except:
                f = open(self._filename,"w")
                f.write("\n")
                f.close()
                f = open(self._filename,"r")
                f2 = f.read()
            self.document.delete("1.0","end")
            self.document.insert("1.0",f2)
            self.document.delete("end-1c","end")
            f.close()
            self.set_title(PERM_TITLE + self._filename)
        self.command_frame.add_item_to_menu(script(self._filename, self))
        self.command_frame.set_option(self._filename.split('.')[0].split(os.sep)[-1])
            
        self.old_text = self.document.get('1.0', 'end')

    def save_file(self,event=None):
        if self.old_text == self.document.get("1.0", "end"):
            return
        if (self._filename == ""):
            self.save_as_file()
        else:
            self.old_text = self.document.get("1.0", "end")
            f = open(self._filename,"w")
            f.write(self.document.get("1.0","end"))
            f.close()
            self.set_title(PERM_TITLE + self._filename)

    def save_as_file(self,event=None):
        #if self.old_text == self.document.get("1.0", "end"):
        #    return
        self.old_text = self.document.get("1.0", "end")
        self._filename = tkFileDialog.asksaveasfilename(filetypes=self._filetypes)
        if not (self._filename == "" or self._filename == None):
            f = open(self._filename,"w")
            f.write(self.document.get("1.0","end"))
            f.close()
            self.set_title(PERM_TITLE + self._filename)
            self.command_frame.add_item_to_menu(script(self._filename, self))
            self.command_frame.set_option(self._filename.split('.')[0].split(os.sep)[-1])

    def minimize(self,event=None):
        self.iconify()

    def charactercount(self,event=None):
        tkMessageBox.showinfo("Character Count","Document contains " + str(len(self.document.get("1.0","end"))-1) + " characters.")

    def options_window(self,event=None):
        optionsbox = OptionsBox(self)
        optionsbox.mainloop()
        
    def has_changes(self):
        try:
            return not self.old_text.rstrip() == self.document.get('1.0', 'end').rstrip()
        except AttributeError:
            return False
        
    def newwindow(self,event=None):
        main(0)

    def destruct(self,event=None):
        self.destroy()
        sys.exit()

    def closewindow(self,event=None):
        self.destroy()
        sys.exit()

    def set_title(self,title=""):
        self.title(title)

    def set_colors(self,
                   bg=_bg,fg=_fg,
                   bg1=_bg1,fg1=_fg1,
                   bg2=_bg2,fg2=_fg2):
        self.tk_setPalette(background=bg,
                           foreground=fg,
                           activeBackground=bg1,
                           activeForeground=fg1,
                           insertBackground=fg,
                           troughColor=bg)
        self.document.configure(bg=bg2,
                                fg=fg2,
                                insertbackground=fg2)
        
    def __init__(self,master=None, root=None, data=None):      # Please read the comments here and
                                         # follow their instructions - this is
                                         # not to limit your freedom, it is
                                         # simply so that you don't accident-
                                         # ally screw up PyWord.
        Tk.__init__(self)
        
        self.old_text = ''
        self.set_title(PERM_TITLE)
        self.geometry('500x500')
        #self.resizable(0,1)  # Do not delete or modify without a
                                         # good reason!
        
        self.create_widgets()            # Do not delete or modify, period!
        self.create_menu() 
        self._filename = ''
        self.change_toolbar_icons(_iconset)
                                         # Cannot be deleted.
        self.set_colors(_bg,_fg,_bg1,_fg1,_bg2,_fg2)
                                         # Do not delete or modify - change
                                         # _bg, _fg, _bg2, and _fg2 at the be-
                                         # ginning of the program if you want
                                         # to change the program's colors.
        self.pack_widgets(1,1,1,"right")
                                         # Modify, but do not delete!  Order
                                         # is (menubar, status bar, scrollbar,
                                         # scrollbar position).
        self.focus_set()                 # This doesn't seem to work... Feel
                                         # free to do whatever the hell you
                                         # want.

###############################################################################
#
# About box
#
###############################################################################

class AboutBox(Toplevel):
    def create_widgets(self):
        fn = fn_standard

        about_text = """
Gibbs
(C) 2009,2010 Thomas Cool
    2009,2010 R. Edwin Garcia
GNU GPL v3.0 Licensed.\n

    
PyText
All rights reserved.\n
(C) 2008 Adam Balan.
    2009 Alex Bartol
All rights reserved.\n
GNU GPL v3.0 Licensed.\n
Pytext is based (source code and all) on the base
of Pyword. PyText allows for the user code in an IDE
for python
Pytext is also a working Text editor.\n
Running in Python %s on %s.\n\
""" % (sys.version[0:3],sys.platform)

        self.about_title = Label(self,
                                 font=fn + " bold",
                                 text="PyWord",
                                 justify="left")
        self.about_title.pack(side="top",expand=1,fill="x")

        self.about_version = Label(self,
                                   font=fn,
                                   text="Version " + __version__)
        self.about_version.pack(side="top",expand=1,fill="x")

        self.about_label = Label(self,
                                 font=fn,
                                 text=about_text,
                                 justify="left")
        self.about_label.pack(side="top",expand=1,fill="both")

        self.ok_frame = Frame(self,
                              borderwidth=2,
                              relief="flat")
        self.ok_frame.pack(side="top",expand=1,fill="x")


        self.ok_button = Button(self.ok_frame,
                                font=fn,
                                relief="raised",
                                default="active",
                                text="   OK   ",
                                command=self.destruct)
        self.ok_button.pack(side="right")

        self.bind("<Return>",self.destruct)
        self.bind("<Escape>",self.destruct)

    def destruct(self,event=None):
        self.destroy()

    def __init__(self,master=None):
        Toplevel.__init__(self,master)
        self.title("About")
        #self.resizable(0,0)
        self.create_widgets()
        self.focus_set()


###############################################################################
##
##        HELP!!!! <- window.....
##
###############################################################################
class HelpBox(Toplevel):
    def create_widgets(self):
        fn = fn_standard

        about_text = """
Using Color ->\n

See the bellow grey box? well
lets state for the fact of
stating that you created some
code in the upper white section.
take that code and paste it into
the bottom box and hit colorize.
it will color code your code.\n

*note: some bugs still being
worked out
""" 

        self.help_title = Label(self,
                                 font=fn + " bold",
                                 text="PyWord",
                                 justify="left")
        self.help_title.pack(side="top",expand=1,fill="x")

        self.help_version = Label(self,
                                   font=fn,
                                   text="Version " + __version__)
        self.help_version.pack(side="top",expand=1,fill="x")

        self.help_label = Label(self,
                                 font=fn,
                                 text=about_text,
                                 justify="left")
        self.help_label.pack(side="top",expand=1,fill="both")

        self.ok_frame = Frame(self,
                              borderwidth=2,
                              relief="flat")
        self.ok_frame.pack(side="top",expand=1,fill="x")

        self.ok_button = Button(self.ok_frame,
                                font=fn,
                                relief="raised",
                                default="active",
                                text="   OK   ",
                                command=self.destruct)
        self.ok_button.pack(side="right")

        self.bind("<Return>",self.destruct)
        self.bind("<Escape>",self.destruct)

    def destruct(self,event=None):
        self.destroy()

    def __init__(self,master=None):
        Toplevel.__init__(self,master)
        self.title("About")
        #self.resizable(0,0)
        self.create_widgets()
        self.focus_set()


###############################################################################
#
# Options Window
#
###############################################################################

class OptionsBox(Toplevel):
    def create_widgets(self):
        fn = fn_standard

        self.scrollbar_frame = Frame(self,
                                     borderwidth=2,
                                     relief="flat")
        self.scrollbar_frame.pack(side="top",expand=1,fill="x")

        self.scrollbar_label = Label(self.scrollbar_frame,
                                     font=fn,
                                     relief="flat",
                                     text="Scrollbar:  ")
        self.scrollbar_label.pack(side="left")

        self.scrollbar_entry = Entry(self.scrollbar_frame,
                                     font=fn,
                                     relief="sunken",
                                     width=10)
        self.scrollbar_entry.pack(side="right",expand=1,fill="x")

        self.scrollbar_entry.insert("0","right")

        self.colors_frame = Frame(self,
                                  borderwidth=2,
                                  relief="flat")
        self.colors_frame.pack(side="top",expand=1,fill="x")

        self.colors_label = Label(self.colors_frame,
                                  font=fn,
                                  relief="flat",
                                  text="Program Colors:  ")
        self.colors_label.pack(side="left")

        self.colors_fg2 = Entry(self.colors_frame,
                                font=fn,
                                relief="sunken",
                                width=7)
        self.colors_fg2.pack(side="right",expand=1,fill="x")

        self.colors_bg2 = Entry(self.colors_frame,
                                font=fn,
                                relief="sunken",
                                width=7)
        self.colors_bg2.pack(side="right",expand=1,fill="x")

        self.colors_fg1 = Entry(self.colors_frame,
                                font=fn,
                                relief="sunken",
                                width=7)
        self.colors_fg1.pack(side="right",expand=1,fill="x")

        self.colors_bg1 = Entry(self.colors_frame,
                                font=fn,
                                relief="sunken",
                                width=7)
        self.colors_bg1.pack(side="right",expand=1,fill="x")

        self.colors_fg = Entry(self.colors_frame,
                               font=fn,
                               relief="sunken",
                               width=7)
        self.colors_fg.pack(side="right",expand=1,fill="x")

        self.colors_bg = Entry(self.colors_frame,
                               font=fn,
                               relief="sunken",
                               width=7)
        self.colors_bg.pack(side="right",expand=1,fill="x")

        self.colors_bg.insert("0",_bg)
        self.colors_fg.insert("0",_fg)
        self.colors_bg1.insert("0",_bg1)
        self.colors_fg1.insert("0",_fg1)
        self.colors_bg2.insert("0",_bg2)
        self.colors_fg2.insert("0",_fg2)

        self.colors_bg.bind("<Return>",self.change_colors)
        self.colors_fg.bind("<Return>",self.change_colors)
        self.colors_bg2.bind("<Return>",self.change_colors)
        self.colors_fg2.bind("<Return>",self.change_colors)

        self.iconset_frame = Frame(self,
                                   borderwidth=2,
                                   relief="flat")
        self.iconset_frame.pack(side="top",expand=1,fill="x")

        self.iconset_label = Label(self.iconset_frame,
                                   font=fn,
                                   relief="flat",
                                   text="Icon Set:  ")
        self.iconset_label.pack(side="left")

        self.iconset = Entry(self.iconset_frame,
                             font=fn,
                             relief="sunken",
                             width=10)
        self.iconset.pack(side="top",expand=1,fill="x")

        self.iconset.insert("0",_iconset)

        self.iconset.bind("<Return>",self.change_icon_set)

        self.wordwrap_frame = Frame(self,
                                    borderwidth=2,
                                    relief="flat")
        self.wordwrap_frame.pack(side="top",expand=1,fill="x")

        self.wordwrap_label = Label(self.wordwrap_frame,
                                    font=fn,
                                    relief="flat",
                                    text="Word Wrap:  ")
        self.wordwrap_label.pack(side="left")

        self.wordwrap = Entry(self.wordwrap_frame,
                              font=fn,
                              relief="sunken",
                              width=10)
        self.wordwrap.pack(side="top",expand=1,fill="x")

        self.wordwrap.insert("0",_wordwrap)

        self.wordwrap.bind("<Return>",self.change_wordwrap)

        
        self.close_button = Button(self.command_frame,
                                   font=fn,
                                   relief="raised",
                                   default="active",
                                   text="     Close     ",
                                   command=self.destruct)
        self.close_button.pack(side="right")
        
        self.bind("<Escape>",self.destruct)

        self.scrollbar_entry.bind("<Return>",self.move_scrollbar)

        self.scrollbar_entry.focus_set()

    def move_scrollbar(self,event=None):
        self.master.document.pack_forget()
        self.master.scrollbar.pack_forget()
        self.master.scrollbar.pack(side=self.scrollbar_entry.get(),expand=1,
                                   fill="y")
        self.master.document.pack(side="top",expand=1,fill="both")

    def change_colors(self,event=None):
        self.master.set_colors(self.colors_bg.get(),self.colors_fg.get(),
                               self.colors_bg1.get(),self.colors_fg1.get(),
                               self.colors_bg2.get(),self.colors_fg2.get())

    def change_icon_set(self,event=None):
        self.master.change_toolbar_icons(self.iconset.get())

    def change_wordwrap(self,event=None):
        self.master.document.configure(wrap=self.wordwrap.get())

    def destruct(self,event=None):
        self.destroy()

    def __init__(self,master=None):
        Toplevel.__init__(self,master)
        self.title("Options")
        #self.resizable(0,0)
        self.create_widgets()
        self.focus_set()


###############################################################################
#
# Are you going to actually run PyWord, or are you importing it so that you
# can, say, create an add-on and modify it?  This part will find out, and, if
# the former case, run the program!
#
###############################################################################

def main(openfile=1, filename=''):

    app = Application()
    bg = _bg
    fg = _fg
    bg2 = _bg2
    fg2 = _fg2

    if openfile:
        for i in range(1, len(sys.argv)):
            app.open_file(None, sys.argv[i])
        if (len(filename) <= 1):
            pass
        else:
            app.open_file(None,filename)



    #if '--noimg' not in sys.argv:
    #    import start
    # The source tools add-on is one which I keep meaning to create (it is
    # supposed to provide syntax highlighting and other useful features).
    # Needless to say, I never started, but you can make one if you want :-)
    #running = True
    #while running:
    #    try:
    app.mainloop()
    #    except KeyboardInterrupt:
    #        pass
    

if __name__ == "__main__":
    if '--title' in sys.argv:
        try:
            PERM_TITLE = sys.argv[sys.argv.index('--title')+1]
        except:
            pass
    main(0)
    
