#!/usr/bin/env python
"""VKMLlive.py
"""

###################################
#VKML live
# Designed by Alex Bartol 
#             Copywright 2009
#   scripts provided by Dr. Edwin Garcia
#
# This program is a frountend for scripts that run with the VKMLgui framework
#
###################################

import os, Tkinter, sys
import tkMessageBox, tkFileDialog

class script:
    """script is a class to handle the <script.py> files
    """
    def __init__(self, filename, pytext, tutorial=None):
        """Initializes the script with the desired filename
        user_defined is a boolean if the user defined it
        
        safety.py is run for all programs
        """
        self.tutorial = tutorial
        self.pytext = pytext
        self.filename = filename
        self.display_name = filename.split('.py')[0].split(os.sep)[-1]
        
    def has_tutorial(self):
        return not self.tutorial == None
    
    def exec_tutorial(self):
        """NOTE: ONLY WORKS ON NANOHUB
        """
        os.system("export %s", self.tutorial)
        
    def run(self):
        self.pytext.run()
        
    def run_tutorial(self):
        if self.has_tutorial():
            self.exec_tutorial()
        
    def edit(self):
        self.pytext.open_file(filename=self.filename)
    
class VKMLframe (Tkinter.Frame):
    def __init__(self, toplevel, pytext_instance, **kwargs):
        Tkinter.Frame.__init__(self, master=toplevel)
#       filenames = os.listdir('scripts')
        scriptSourceDirectory = os.path.join(os.path.dirname(sys.argv[0]),'scripts')
        filenames = os.listdir(scriptSourceDirectory)
        self.pytext = pytext_instance
        self.scripts = []
        
        #Add txt, gbs, and tbs files to tmp
#        os.system('cp scripts/*.tdb /tmp')
#        os.system('cp scripts/*.txt /tmp')
#        os.system('cp scripts/*.gbs /tmp')
        if '--nanohub' in sys.argv:
            os.system('cp /apps/gibbs/dev/bin/scripts/*.tdb /tmp')
            os.system('cp /apps/gibbs/dev/bin/scripts/*.txt /tmp')
            os.system('cp /apps/gibbs/dev/bin/scripts/*.gbs /tmp')
        else:
            os.system("cp ./scripts/* /tmp")
        
        for fle in filenames:
            if fle.endswith('.py') or fle.endswith('.pyc'):
                scriptSource = os.path.join(scriptSourceDirectory,fle)
                os.system('cp ' + scriptSource + ' /tmp')
#               fle = os.curdir + os.sep + 'scripts' + os.sep + fle
                fle = os.path.join('/tmp',fle)
                self.scripts.append(script(fle, self.pytext))
        self.create_gui()
        #os.system('cp  ./gui/*.py /tmp/gui')
    def add_item_to_menu(self, script):
        """Adds item 'item' to the dropdown list
        """
        for s in self.scripts:
            if s.display_name == script.display_name:
                return
        self.scripts.append(script)
        self.options.insert(3, self.scripts[-1].display_name)
        self.w['menu'].insert(3, 'command', label=script.display_name, command=lambda : self.var.set(script.display_name))
         
    def set_option(self, string):
        #print string
        if string in self.options:
            self.var.set(string)   
            self.current = self.var.get()    
            
    def create_gui(self):
        """creates, configures, and starts up the GUI
        scripts is a list of class type script
        """
        top = self
        
        self.options = []
        self.w = None
        self.options.append('SELECT AN OPTION')
        self.options.append('----------------')
        self.current = self.options[0]
        
        
                
        def open_file(event=None):
            f = tkFileDialog.askopenfile()
            filename = f.name
            f.close()
            scripts.append(script(filename, self.pytext))
            #options.append(scripts[-1].display_name)
            add_item_to_menu(scripts[-1].display_name)
            self.var.set(scripts[-1].display_name)
            #pytext.main(filename=filename)
            
        
        
        for s in self.scripts:
            self.options.append(s.display_name)
        self.options.append('----------------')
        self.options.append('Open File')
        self.options.append('New File')
        self.var = Tkinter.StringVar(top)
        self.var.set(self.options[0])
        self.w = apply(Tkinter.OptionMenu, (top, self.var) + tuple(self.options))


        def run(try_again=False):
            for x in self.scripts:
                if self.var.get() == x.display_name:
                    if self.pytext.has_changes():
                        answer = tkMessageBox.askyesno(message='Would you like to save first?', title='Save?')
                        if answer == True:
                            self.pytext.save_file()
                    x.run()
                    return
            if not try_again:
                self.pytext.save_as_file()
                run(True)
        def edit(a=None, b=None, c=None):
            for x in self.scripts:
                if self.var.get() == x.display_name:
                    if self.pytext.has_changes():
                        answer = tkMessageBox.askyesno(message='Save before switching?', title='Save?')
                        if answer == None:
                            self.var.set(self.current)
                            return
                        if answer == True:
                            self.pytext.save_file()
                        self.current = self.var.get()
                        x.edit()
                        return
                    else:
                        self.current = self.var.get()
                        x.edit()
                        return
            if self.var.get() == 'Open File':
                self.pytext.open_file()
                self.current = self.pytext._filename
                return
            elif self.var.get() == 'New File':
                self.pytext.new_file()
                #self.current = self.pytext._filename
                
                
            #self.var.set(self.current)
        self.var.trace_variable('w', edit)
        def tutorial():
        	for x in self.scripts:
        	    if self.var.get() == x.display_name:
        	        x.run_tutorial()
        self.w.pack(padx=10, pady=10, side=Tkinter.LEFT)
        
        tutorial_button = Tkinter.Button(top, text='Tutorial', command=tutorial)
        tutorial_button.pack(padx=10, pady=10, side=Tkinter.LEFT)
        
        run_button = Tkinter.Button(top, text='Run', command=run)
        run_button.pack(padx=10, pady=10, side=Tkinter.LEFT)
