#
#  guiPhaseSolverC.py
#  gibbs
#
#  Created by Thomas on 9/25/09.
#  Copyright 2009 Thomas Cool, R. Edwin Garcia. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.
#
# example:  creates a small gui in vkmllive, creates a solver and runs it
#           uses the Semi-Regular free energy generator found in QHSolve
#           draws the free Energy Lines and the common tangents

import VKML.gui as VKMLgui

from gibbs.variables import RegularFreeEnergyVariable as RFE
from gibbs.solvers.qhsolver.qhbinarysolver import CommonTangentSolver
from gibbs.viewers import MatplotlibViewer

MatplotlibViewer= MatplotlibViewer(title='Temperature', limits=[-0.05, 1.05, None, None])

def callback(aspectRatio, omega_one, omega_two, Temperature):

    DG1=RFE(omega=omega_one, delta_Ha=8000, delta_Hb=12000, Tma=800, Tmb=1200)
    DG2=RFE(omega=omega_two, delta_Ha=0, delta_Hb=0, Tma=800, Tmb=1200)
    

    #pass them in a list
    Solver=CommonTangentSolver(DG=[DG1, DG2],
               x=[0, 1, 300],
               y=[Temperature])

    #display the phase diagram
    MatplotlibViewer.plot(vars=Solver, aspectRatio=aspectRatio)
    MatplotlibViewer.save('1.eps', overwrite=True)

	
    
if __name__ == '__main__':
    p = VKMLgui.Parser(title='QHSolver')
    m1 = VKMLgui.Menu(title='x Properties', parser=p)
    VKMLgui.Parameter(name='Temperature', menu=m1, variable=int, default=850, interval=(0, 2000))
    VKMLgui.Parameter(name='omega_one', menu=m1, variable=int, default=0, interval=(-30000, 30000))
    VKMLgui.Parameter(name='omega_two', menu=m1, variable=int, default = 10000, interval=(-30000, 30000))
    VKMLgui.Parameter(name='aspectRatio', menu=m1, variable=float, default=1., interval=(0.1,10))

    p.add_command(callback)
    p.set_credits('''
    Script 
    Created by:
    Thomas Cool
    R. Edwin Garcia
    Copyright 2009,2010
    Purdue University
    
    Gibbs 
    Created by:
    Thomas Cool
    R. Edwin Garcia
    Copyright 2009,2010
    Purdue University
    ''')

    p()
