#
#  ConvexHull+python+GUI+matplotlib_gnuplot.py
#  gibbs
#
#  Created by Thomas Cool on 9/19/09.
#  Copyright 2009 Thomas Cool, R. Edwin Garcia. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.
#
# Simple Example using sympy to compute the freeEnergies
# in this case sympy is the time limiting factor
# in DG equations Temperature has to be replaced by Temp and 
# x is the coordinate accross the phase diagram

from gibbs.variables import PyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import CommonTangentSolver
from gibbs.tools.functions import xlogx
from gibbs.viewers import GnuplotViewer,MatplotlibViewer
import VKML.gui as VKMLgui
    
MatplotlibV=MatplotlibViewer(title='M', mode='show')
GnuplotV=GnuplotViewer(title='G')

def callback(Temp=1300, viewer='gnuplot'):
    TempV=Temp
    x=PyVariable('x')
    R=PyVariable('R')
    Temp=PyVariable('Temp')
    omega=PyVariable('omega')
    delta_Ha=PyVariable('delta_Ha')
    delta_Hb=PyVariable('delta_Hb')
    Tma=PyVariable('Tma')
    Tmb=PyVariable('Tmb')
    
    #free energy functions
    DG1=PyVariable(omega * x * (1.0 - x) + R * Temp * (xlogx(x) + xlogx(1.0 - x)) + (1.0 - x) * (delta_Ha - Temp * delta_Ha / Tma) + x * (delta_Hb - Temp * delta_Hb / Tmb))
    DG2=PyVariable(omega * x * (1.0 - x) + R * Temp * (xlogx(x) + xlogx(1.0 - x)) + (1.0 - x) * (delta_Ha - Temp * delta_Ha / Tma) + x * (delta_Hb - Temp * delta_Hb / Tmb))

    #Plug in Variables
    DG1(omega=10000, R=8.314, delta_Ha=8000, delta_Hb=12000, Tma=800, Tmb=1200)
    DG2(omega=15000, R=8.314, delta_Ha=0,    delta_Hb=0,     Tma=800, Tmb=1200)


    #solve for the free Energies
    Solver=CommonTangentSolver(DG=[DG1, DG2], variables=[x, Temp], x=[0, 1, 150], y=[TempV])

    #display the phase diagram
    if viewer == 'matplotlib':
        MatplotlibV.plot(vars=Solver)
    else:
        GnuplotV.plot(vars=Solver)
    
if __name__ == '__main__':
        Temp = 1000
        viewer = 'matplotlib'
        p = VKMLgui.Parser(title='CommonTangent')
        m1 = VKMLgui.Menu(title='Properties', parser=p)
        VKMLgui.Parameter(name='Temp', menu=m1, variable=int, default=Temp, interval=(0, 3000))
        VKMLgui.Parameter(name='viewer', menu=m1, variable=tuple, default=('matplotlib', 'gnuplot'))

        p.add_command(callback)
        p.set_credits('''
    Script 
    Created by:
    Thomas Cool
    R. Edwin Garcia
    Copyright 2009,2010
    Purdue University
    
    Gibbs 
    Created by:
    Thomas Cool
    R. Edwin Garcia
    Copyright 2009,2010
    Purdue University
    ''')
        p()
