#
#  ConvexHull+python+gnuplot.py
#  gibbs
#
#  Created by Thomas Cool on 9/19/09.
#  Copyright 2009 Thomas Cool, R. Edwin Garcia. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gibbs.  If not, see <http://www.gnu.org/licenses/>.
#
# Simple Example using sympy to compute the freeEnergies
# in this case sympy is the time limiting factor
# in DG equations Temperature has to be replaced by Temp and 
# x is the coordinate accross the phase diagram

from gibbs.variables import PyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import CommonTangentSolver
from gibbs.tools.functions import xlogx
from gibbs.viewers import GnuplotViewer

#define variables for free energy equations
x=PyVariable('x')
R=PyVariable('R')
Temp=PyVariable('Temp')
omega=PyVariable('omega')
delta_Ha=PyVariable('delta_Ha')
delta_Hb=PyVariable('delta_Hb')
Tma=PyVariable('Tma')
Tmb=PyVariable('Tmb')

#free energy functions
DG1=PyVariable(omega * x * (1.0 - x) + R * Temp * (xlogx(x) + xlogx(1.0 - x)) + (1.0 - x) * (delta_Ha - Temp * delta_Ha / Tma) + x * (delta_Hb - Temp * delta_Hb / Tmb))
DG2=PyVariable(omega * x * (1.0 - x) + R * Temp * (xlogx(x) + xlogx(1.0 - x)) + (1.0 - x) * (delta_Ha - Temp * delta_Ha / Tma) + x * (delta_Hb - Temp * delta_Hb / Tmb))

#simplify free energy functions
DG1(omega=10000, R=8.314, delta_Ha=8000, delta_Hb=12000, Tma=800, Tmb=1200)
DG2(omega=15000, R=8.314, delta_Ha=0,    delta_Hb=0,     Tma=800, Tmb=1200)

#init + solve (very slow) -- using sympy
Solver=CommonTangentSolver(DG=[DG1, DG2],
                variables = [x,Temp],
                x=[0, 1, 150],
                y=[1000])
Solver.write('freeEnergyData.txt')

#display the phase diagram

Viewer=GnuplotViewer(vars=Solver)
Viewer.plot()
