#
#
#  PhaseDiagram+C+loop+matplotlib_gnuplot.py
#  gibbs
#
#  Created by Thomas Cool on 10/14/09.
#  Copyright 2009 Thomas Cool, R. Edwin Garcia . All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.

#

# example:  same as PhaseDiagram+C+loop+matplotlib_gnuplot.py
#           please refer to that file
#           except it also saves the images generated

from gibbs.variables import RegularFreeEnergyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver
from gibbs.viewers import GnuplotViewer,MatplotlibViewer

def loop(viewer='matplotlib'):
    if viewer not in ['gnuplot','matplotlib']:
        viewer = 'gnuplot'
    
    if viewer == 'gnuplot':
        Viewer = GnuplotViewer(title='Regular C')
    elif viewer == 'matplotlib':
        Viewer = MatplotlibViewer()
        
    for i in range(500):
        #Define equivalent Regular Energy
        DG1 = RegularFreeEnergyVariable(omega=(120 * i) - 30000, 
                              delta_Ha=8000, delta_Hb=12000, Tma=800, Tmb=1200)
        
        DG2 = RegularFreeEnergyVariable(omega=15000, delta_Ha=0, delta_Hb=0, 
                                                             Tma=800, Tmb=1200)
        

     
        bb = PhaseDiagramSolver(DG=[DG1, DG2],
                   x=[0, 1, 400],
                   y=[140, 1800, 500])
        
        Viewer.plot(vars=bb, title='$\Omega1=%i$'%((120 * i) - 30000), xlabel='%B',
                    ylabel='Temperature $(\degree C)$', xmode='percent')
        
        #saving the image generated by the viewer
        Viewer.save(file='temp/Image.png')

    
if __name__ == '__main__':
    loop(viewer='matplotlib')

