#
#
#  PhaseDiagram+C+loop+matplotlib_gnuplot.py
#  gibbs
#
#  Created by Thomas Cool on 10/14/09.
#  Copyright 2009 Thomas Cool, R. Edwin Garcia. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.

#

# example:  Does not create a gui
#           Creates 2 regular free energies and iterates 1000 times over a 
#           varying omega for the first free energy

from gibbs.variables import RegularFreeEnergyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver
from gibbs.viewers import GnuplotViewer,MatplotlibViewer

def loop(viewer='gnuplot'):
    if viewer not in ['gnuplot','matplotlib']:
        viewer='gnuplot'
    
    if viewer == 'gnuplot':
        Viewer=GnuplotViewer(title='Regular C')
    elif viewer == 'matplotlib':
        Viewer=MatplotlibViewer(title='Regular C')
        
    for i in range(1000):
        
        # define equivalent Regular Energy
        DG1=RegularFreeEnergyVariable(omega=(60 * i) - 30000, delta_Ha=8000,
                                       delta_Hb=12000, Tma=800, Tmb=1200)
        
        DG2=RegularFreeEnergyVariable(omega=15000, delta_Ha=0, 
                                      delta_Hb=0, Tma=800, Tmb=1200)
        

     
        Solver=PhaseDiagramSolver(DG=[DG1, DG2],
                   x=[0, 1, 200],
                   y=[180, 1700, 300])

        Viewer.plot(vars=Solver)
    
if __name__=='__main__':
    loop(viewer='gnuplot')

