#
#
#  PhaseDiagram+CompoundVariables+gui+gnuplot.py
#  gibbs
#
#  Created by Thomas Cool on 10/14/09.
#  Copyright 2009 Thomas Cool. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.

#

# example:  creates a small gui in vkmllive, creates a solver and runs it
#           uses the Pyvariables to generate enthalpic interactions and adds that to an idealfreeenergy
#           It is also compared to a regularfreeenergy


import VKML.gui as VKMLgui

from gibbs.variables import PyVariable,IdealFreeEnergyVariable,RegularFreeEnergyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver
from gibbs.viewers.gnuplot import GnuplotViewer

CompoundViewer          = GnuplotViewer(title='With IdealFreeEnergy C + Enthalpic Term in Python')
RegularFreeEnergyViewer = GnuplotViewer(title='With RegularFreeEnergy C')

def callback(omega_one=10000, omega_two=20000):
    #define symbols for python variable
    x     = PyVariable('x')
    omega = PyVariable('omega')
    Temp  = PyVariable('Temp')
    
    DG1_add = PyVariable(omega * x * (1.0 - x))
    
    DG1_add(omega=omega_one)
    DG1_id = IdealFreeEnergyVariable(delta_Ha=8000, delta_Hb=12000, Tma=800, Tmb=1200)
    
    #Add two parts of free energy together
    DG1 = DG1_id + DG1_add
    
    #Define equivalent Regular Energy
    DG2 = RegularFreeEnergyVariable(omega=omega_one, delta_Ha=8000, delta_Hb=12000, Tma=800, Tmb=1200)
    DG3 = RegularFreeEnergyVariable(omega=omega_two, delta_Ha=0,    delta_Hb=0,     Tma=800, Tmb=1200)
    

    #pass them in a list
    CompoundSolver=PhaseDiagramSolver(DG=[DG1, DG3],
               variables = [x, Temp],
               x=[0, 1, 200],         #xmin, xmax, xsteps
               y=[300, 1400, 400])    #ymin, ymax, ysteps
               
    RegularFreeEnergySolver=PhaseDiagramSolver(DG=[DG2, DG3],
               x=[0, 1, 200],
               y=[300, 1400, 400])


    #display the phase diagram
    CompoundViewer.plot(vars=CompoundSolver)
    RegularFreeEnergyViewer.plot(vars=RegularFreeEnergySolver)
    
if __name__ == '__main__':
    p = VKMLgui.Parser(title='QHSolver', interactive=-1)
    m1 = VKMLgui.Menu(title='x Properties', parser=p)
    VKMLgui.Parameter(name='omega_one', menu=m1, variable=int, default=10000, interval=(-30000, 30000))
    VKMLgui.Parameter(name='omega_two', menu=m1, variable=int, default=20000, interval=(-30000, 30000))

    p.add_command(callback)
    p.set_credits('''
    Script 
    Created by:
    Thomas Cool
    R. Edwin Garcia
    Copyright 2009,2010
    Purdue University
    
    Gibbs 
    Created by:
    Thomas Cool
    R. Edwin Garcia
    Copyright 2009,2010
    Purdue University
    ''')
    p()
