#    PhaseDiagram_ConvexHull+ThermoCalc+matplotlib.py
#    gibbs
#
#
#    Created by Thomas Cool on 28-Jan-2010.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.
#

from gibbs.viewers import MatplotlibViewer
from gibbs.variables import TCDBVariable, PyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver,CommonTangentSolver
from time import time

#Load a database
DB = TCDBVariable('sncuag.tdb')

#select the Phases...
a = DB.setPhase('FCC_A1')
b = DB.setPhase('LIQUID')
#print a.Phases()
#..and the components corresponding to the phases
#they have to be set after setting the phase because their availability depends
#on them being defined for that phase in the database
#changing the phases removes the set components too
#If a phase only has 2 components defined, then they will automatically be selected
#to reverse the order of the Phase Diagram, the order of the components needs to be changed
print(DB.Components())
print(a.Components())
a.setComponents(["AG", "CU"])
b.setComponents(["AG", "CU"])

t1 = time()

solver1 = PhaseDiagramSolver(DG=[a, b], x=[0, 1, 400], y=[400, 1500, 800])

t2 = time()

solver2 = CommonTangentSolver(DG=[a, b], x=[0, 1, 400], y=[1100])

print('Time to solve: %.2f seconds for 800 temperature samples'%(t2-t1))

viewer1 = MatplotlibViewer(vars=solver1, title='Ag-Cu Phase Diagram')
viewer2 = MatplotlibViewer(vars=solver2, title='Gibbs Free Energy at T=1100K for Ag-Cu')

viewer1.plot(xlabel='%Cu', figureNumber=1, ylabel='T (K)',      xmode='percent')
viewer2.plot(xlabel='%Cu', figureNumber=2, ylabel='$\Delta G$', xmode='percent')

#Nothing stops anyone from adding a PyVariable to a TCDBVariable as all the others

stress=PyVariable('stress')
x = PyVariable('x')
temp = PyVariable('temp')
stresscontrib = PyVariable(10 * stress * x)
stresscontrib(stress=200)

solver3 = CommonTangentSolver(DG=[a + stresscontrib, b],x=[0, 1, 400],y=[1100], variables=[x, temp])
viewer3 = MatplotlibViewer(vars=solver3, title='Gibbs Free Energy at T=1100K for Ag-Cu + extra contrib')
viewer3.plot(xlabel='%Cu', figureNumber=3, ylabel='$\Delta G$', 
             xmode='percent', mode='show')

