#!/usr/bin/env python
#    ferroelectricsAnalytics.py
#    gibbs
#
#
#    Created by R. Edwin Garcia on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.




import time
import fipy.tools.numerix as numerix
from fipy.terms.implicitDiffusionTerm import ImplicitDiffusionTerm
from fipy.terms.transientTerm import TransientTerm
from fipy.solvers.pysparse.linearLUSolver import LinearLUSolver
##from fipy.solvers.trilinos.linearGMRESSolver import LinearGMRESSolver

start = time.time()
numberOfElements = 64000 
steps = 550

nx = int(numerix.sqrt(numberOfElements))
ny = int(numerix.sqrt(numberOfElements))
dx = 1.0/float(nx)
dy = 1.0/float(ny)



kappa = 1e-4
M = 1.0

from fipy.meshes.grid2D import Grid2D
mesh = Grid2D(dx=dx, dy=dy, nx=nx, ny=ny)

from fipy.variables.cellVariable import CellVariable
from fipy.tools.numerix import random

P = 1.0

var = CellVariable(name='phase field',
                   mesh=mesh,
                   value=2.0 * P * random.random(nx * ny) - P)


from fipy.tools.numerix import sin
## ANALYTICS
from gibbs.variables import PyVariable
P = PyVariable('P')
a = PyVariable('a')
b = PyVariable('b')
c = PyVariable('c')
EField = PyVariable('EField')


F = (a / 2.0) * (P ** 2) + (b / 4.0) * (P ** 4) + (c/6.0) * (P ** 6) - (EField * P)
##OLD LINE
#F = (a/2.0) * P**2 + (b/4.0) * P**4 + (c/6.0) * P**6 - 1*EField*P

dfdP = PyVariable(F.diff(P)) ## take derivative
print('dFdP=', dfdP)

dfdPfipy = dfdP(a=-1.0, b=1.0, c=0.0, P=var, EField=0)

doubleWellDerivative = M * dfdPfipy

gamma = M * kappa
diffTerm = ImplicitDiffusionTerm(coeff=M * kappa)
eqch = TransientTerm() - diffTerm + doubleWellDerivative



solver = LinearLUSolver(tolerance=1e-7, steps=1000)
##solver = LinearGMRESSolver(tolerance = 1e-7,steps = 1000)


from fipy.boundaryConditions.fixedValue import FixedValue
from fipy.boundaryConditions.fixedFlux import FixedFlux
from fipy.boundaryConditions.nthOrderBoundaryCondition import NthOrderBoundaryCondition
BCs = (FixedFlux(mesh.getFacesRight(), 0),
       FixedFlux(mesh.getFacesLeft(), 0),
       FixedFlux(mesh.getFacesTop(), 0),
       FixedFlux(mesh.getFacesBottom(), 0) )

if __name__ == '__main__':

    import fipy.viewers
    viewer = fipy.viewers.make(vars=var, limits={'datamin' : -1.0, 
                                                 'datamax': 1.0})
    viewer.plot()
    dexp = -5
    for step in range(steps):
        dt = numerix.exp(dexp)
        dt = min(100, dt)
        dexp += 0.01
        var.updateOld()
        eqch.solve(var, boundaryConditions=BCs, solver=solver, dt=dt)
        viewer.plot()
        print('step', step, 'dt', dt)


            
