#
#  variableCreation.py
#  gibbs
#
#  Created by Thomas Cool on 9/19/09.
#  Copyright 2009 Thomas Cool, R. Edwin Garcia. All rights reserved.
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.

#Shows how to create examples


from gibbs.variables import IdealFreeEnergyVariable, PyVariable, RegularFreeEnergyVariable

#creating an ideal free energy variable:
print('\ncreating an IdealFreeEnergyVariable with name ideal and parameters: \
       \ndelta_Ha=%i, delta_Hb=%i, Tma=%i, Tmb=%i'%(8000, 12000, 800, 1200))
       
ideal = IdealFreeEnergyVariable(delta_Ha=8000,
                                delta_Hb=12000,
                                Tma=800,
                                Tmb=1200)
                                
print('printing ideal: %s' % ideal)
                                
#Python Variables Require a bit more work
#you first need to create the "variable's variables"
print('creation of Python Variable (PyVariable)')
print('create x=PyVariable(\'x\') and Omega=PyVariable(\'Omega\')')
x = PyVariable('x')

Omega = PyVariable('Omega')
print('\nCombine them in PyVariable form: PythonTerm=PyVariable(Omega*x*(1.0-x))')

PythonTerm = PyVariable(Omega * x * (1.0 - x))
print('\nPythonTerm: %s\n' % PythonTerm)

#It is also possible to replace omega at two different times:
#   if you pass Omega in the variables Parameter of the Solver, it will be a 
#       temporary replacement (duration of the Hull/Phase Diagram)
#   if you pass Omega now, it will be a permanent replacement for the duration
#       of the existence of the Variable:
print('\nReplacing Omega with 10000 in PythonTerm:\nPythonTerm(Omega=10000)')
PythonTerm(Omega=10000) # This replaces Omega permamently with 10000
print('\nPythonTerm: %s\n' % PythonTerm)

#We can also create built in regular free energy variables
print('\nCreating a RegularFreeEnergyVariable with name ideal and parameters: \n\
      omega=%i delta_Ha=%i, delta_Hb=%i, Tma=%i, Tmb=%i'%(10000, 0, 0, 800, 1200))
print('regular=RegularFreeEnergyVariable(omega=10000,delta_Ha=0,delta_Hb=0,\
                                         Tma=800,Tmb=1200)')
regular = RegularFreeEnergyVariable(omega=10000, delta_Ha=0, delta_Hb=0, Tma=800, Tmb=1200)
print('\nregular: %s\n' % regular)

#Finally The Variables can also be added together:
print('\nadding Variables PythonTerm and ideal into sumVar\nsumVar=ideal+PythonTerm')

sumVar = ideal + PythonTerm
print('\nsumVar: %s' % sumVar)
print('sumVar and regular should now be the equivalent...')



