#
#  PhaseDiagram+C+GUI.py
#  gibbs
#
#  Created by Thomas on 9/28/09.
#  Copyright (c) 2009 Thomas Cool. All rights reserved.
#
# example:  creates a small gui in vkmllive, creates a solver and runs it
#           uses the Semi-Regular free energy generator found in QHSolve
#           draws the Phase diagram for those phases


from gibbs.variables import RegularFreeEnergyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver 
from gibbs.viewers.gnuplot import GnuplotViewer

#can define multiple options :

Viewer= GnuplotViewer(title='Regular Free Energy')
def printPhase(omega_1,omega_2,min_Y,max_Y):
    
    # First Phase
    DG1=RegularFreeEnergyVariable(omega=omega_1,delta_Ha=8000,delta_Hb=12000,Tma=800,Tmb=1200)
    
    # Second Phase
    DG2=RegularFreeEnergyVariable(omega=omega_2,delta_Ha=0,delta_Hb=0,Tma=800,Tmb=1200)

    #Create a Solver
    Solver=PhaseDiagramSolver(DG=[DG1,DG2],
                                x=[0,1,200],            #defines range for x-axis   
                                y=[min_Y,max_Y,200],    #defines range for y-axis
                                PB='None')              #define progress bar type
                                
    #Plot the computed Data in the previously created solver
    Viewer.plot(vars=Solver)

if __name__=='__main__':
    from gibbs.tools.gui import VKMLgui

    p=VKMLgui.Parser(title='vary',interactive=1)
    m1=VKMLgui.Menu(title='Menu',parser=p)
    VKMLgui.Parameter(name='min_Y',menu=m1,variable=int,default=300,interval=(0,4000))
    VKMLgui.Parameter(name='max_Y',menu=m1,variable=int,default=2000,interval=(0,4000))
    VKMLgui.Parameter(name='omega_1',menu=m1,variable=int,default=15000,interval=(-30000,30000))
    VKMLgui.Parameter(name='omega_2',menu=m1,variable=int,default=15000,interval=(-30000,30000))
    p.add_command(printPhase)
    p.run()
