#
#
#  PhaseDiagram+CompoundVariables+gui+gnuplot.py
#  gibbs
#
#  Created by Thomas Cool on 10/14/09.
#  Copyright 2009 Thomas Cool. All rights reserved.
#

# example:  creates a small gui in vkmllive, creates a solver and runs it
#           uses the Pyvariables to generate enthalpic interactions and adds that to an idealfreeenergy
#           It is also compared to a regularfreeenergy

from gibbs.tools.gui import VKMLgui



from gibbs.variables import PyVariable,IdealFreeEnergyVariable,RegularFreeEnergyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver
from gibbs.viewers.gnuplot import GnuplotViewer

CompoundViewer          = GnuplotViewer(title='With IdealFreeEnergy C + Enthalpic Term in Python')
RegularFreeEnergyViewer = GnuplotViewer(title='With RegularFreeEnergy C')

def callback(omega_one=10000,omega_two=20000):
    x=PyVariable('x')                 #define symbols for python variable
    omega=PyVariable('omega')
    Temp=PyVariable('Temp')
    
    DG1_add=PyVariable(omega*x*(1.0-x)) #set up python variable
    
    DG1_add(omega=omega_one)                #define Omega
    DG1_id=IdealFreeEnergyVariable(delta_Ha=8000,delta_Hb=12000,Tma=800,Tmb=1200) #define Ideal Free Energy
    DG1 = DG1_id + DG1_add     #Add two parts of free energies together
    
    DG2=RegularFreeEnergyVariable(omega=omega_one,delta_Ha=8000,delta_Hb=12000,Tma=800,Tmb=1200) # define equivalent Regular Energy
    
    DG3=RegularFreeEnergyVariable(omega=omega_two,delta_Ha=0,delta_Hb=0,Tma=800,Tmb=1200)
    

    #pass them in a list
    CompoundSolver=PhaseDiagramSolver(DG=[DG1, DG3],
               variables = [x,Temp],
               x=[0,1,200],         #xmin, xmax, xsteps
               y=[300,1400,400])    #ymin, ymax, ysteps
               
    RegularFreeEnergySolver=PhaseDiagramSolver(DG=[DG2,DG3],
           # variables = [x, Temp], #variables is only necessary when a pyvariable is used, therefore it is not necessary now
               x=[0,1,200],
               y=[300,1400,400])


    #display the phase diagram
    
    CompoundViewer.plot(vars=CompoundSolver)
    RegularFreeEnergyViewer.plot(vars=RegularFreeEnergySolver)
    
if __name__=='__main__':
    p=VKMLgui.Parser(title='QHSolver',interactive=-1)
    m1=VKMLgui.Menu(title='x Properties',parser=p)
    VKMLgui.Parameter(name='omega_one',menu=m1,variable=int,default = 10000, interval=(-30000,30000))
    VKMLgui.Parameter(name='omega_two',menu=m1,variable=int,default = 20000, interval=(-30000,30000))


    p.add_command(callback)



    p.run()
