#
#  PhaseDiagram+Python+gnuplot.py
#  Gibbs
#
#  Created by Thomas Cool on 9/19/09.
#  Copyright (c) 2009 Thomas Cool. All rights reserved.
#
# Creates a phase diagram using Python to compute the free energies 

from gibbs.variables import PyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import CommonTangentSolver,PhaseDiagramSolver
from gibbs.tools.functions import xlogx
from gibbs.viewers import GnuplotViewer
import time
#define variables for free energy equations
x=PyVariable('x')
R=PyVariable('R')
Temp=PyVariable('Temp')
omega=PyVariable('omega')
delta_Ha=PyVariable('delta_Ha')
delta_Hb=PyVariable('delta_Hb')
Tma=PyVariable('Tma')
Tmb=PyVariable('Tmb')

t1=time.time()
#free energy functions
DG1=PyVariable(omega*x*(1.0-x)+R*Temp*(xlogx(x)+xlogx(1.0-x))+(1.0-x)*(delta_Ha-Temp*delta_Ha/Tma)+x*(delta_Hb-Temp*delta_Hb/Tmb))
DG2=PyVariable(omega*x*(1.0-x)+R*Temp*(xlogx(x)+xlogx(1.0-x))+(1.0-x)*(delta_Ha-Temp*delta_Ha/Tma)+x*(delta_Hb-Temp*delta_Hb/Tmb))

#Plug in values
DG1(omega=10000,R=8.314,delta_Ha=8000,delta_Hb=12000,Tma=800,Tmb=1200)
DG2(R=8.314,omega=15000,delta_Ha=0,delta_Hb=0,Tma=800,Tmb=1200)


#init + solve (very slow) -- using sympy
Solver=PhaseDiagramSolver(DG=[DG1,DG2],         #free energies
                        variables=['x',Temp],   #variable on x-axis, variable on y-axis can be based as either a string or a Pyvariable
                        x=[0,1,200],            #xmin xmax xsteps
                        y=[300,1400,300],       #ymin ymax ysteps
                        ProgressBar='TK')       #progressbar style
print "time: ",t1-time.time()
Viewer=GnuplotViewer(vars=Solver)
Viewer.plot()

#display the phase diagram
