#
#  PhaseDiagram+python+matplotlib.py
#  gibbs
#
#  Created by Thomas Cool on 9/19/09.
#  Copyright (c) 2009 Thomas Cool. All rights reserved.
#
# same as PhaseDiagram+python+gnuplot.py
# except it uses a different viewer


from gibbs.variables import PyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver
from gibbs.tools.functions import xlogx
from gibbs.viewers import MatplotlibViewer

#define variables for free energy equations
x=PyVariable('x')
R=PyVariable('R')
Temp=PyVariable('Temp')
omega=PyVariable('omega')
delta_Ha=PyVariable('delta_Ha')
delta_Hb=PyVariable('delta_Hb')
Tma=PyVariable('Tma')
Tmb=PyVariable('Tmb')


#free energy functions
DG1=PyVariable(omega*x*(1.0-x)+R*Temp*(xlogx(x)+xlogx(1.0-x))+(1.0-x)*(delta_Ha-Temp*delta_Ha/Tma)+x*(delta_Hb-Temp*delta_Hb/Tmb))
DG2=PyVariable(omega*x*(1.0-x)+R*Temp*(xlogx(x)+xlogx(1.0-x))+(1.0-x)*(delta_Ha-Temp*delta_Ha/Tma)+x*(delta_Hb-Temp*delta_Hb/Tmb))


#simplify free energy functions
DG1(omega=10000,R=8.314,delta_Ha=8000,delta_Hb=12000,Tma=800,Tmb=1200)
DG2(R=8.314,omega=15000,delta_Ha=0,delta_Hb=0,Tma=800,Tmb=1200)

Solver=PhaseDiagramSolver(DG=[DG1, DG2],
        variables = [x, Temp], 
        x=[0,1,200],
        y=[300, 1350, 400],
        ProgressBar='TK')


#display the phase diagram

Viewer=MatplotlibViewer()
Viewer.plot(vars=Solver,mode='show') #matplotlib needs mode='show' if you are showing just one figure 
