#
#  guiCPhaseDiagram.py
#  gibbs
#
#  Created by Thomas on 9/28/09.
#  Copyright (c) 2009 Thomas Cool. All rights reserved.
#
# example:  creates a small gui in vkmllive, creates a solver and runs it
#           uses the Semi-Regular free energy generator found in QHSolve
#           draws the Phase diagram for those phases and the common tangents

from gibbs.variables import RegularFreeEnergyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import CommonTangentSolver
from gibbs.solvers.qhsolver.qhbinarysolver import PhaseDiagramSolver 
from gibbs.viewers.gnuplot import GnuplotViewer
import time

#can define multiple options :

Viewer2= GnuplotViewer(title='Common Tangent')
Viewer1= GnuplotViewer(title='Phase Diagram')

def printPhase(omega_1,omega_2,min_Y,max_Y):
    DG1=RegularFreeEnergyVariable(omega=omega_1,delta_Ha=8000,delta_Hb=12000,Tma=800,Tmb=1200)
    DG2=RegularFreeEnergyVariable(omega=omega_2,delta_Ha=0,delta_Hb=0,Tma=800,Tmb=1200)
    variables=[['%B'],['Temp']]
    Solver = PhaseDiagramSolver(DG=[DG1,DG2],variables = variables,x=[0,1,200],y=[min_Y,max_Y,200])
    Viewer1.plot(vars=Solver)

def printCommonTangent(omega_1,omega_2,temperature,min_Y,max_Y):
    DG1=RegularFreeEnergyVariable(omega=omega_1,delta_Ha=8000,delta_Hb=12000,Tma=800,Tmb=1200)
    DG2=RegularFreeEnergyVariable(omega=omega_2,delta_Ha=0,delta_Hb=0,Tma=800,Tmb=1200)
    variables=[['%B'],['DG']]
    Solver  = CommonTangentSolver(DG=[DG1,DG2],variables = variables,x=[0,1,200],y=[temperature])
    Viewer2.plot(vars=Solver)

#solve
#print
    
from gibbs.tools.gui import VKMLgui

p=VKMLgui.Parser(title='vary',interactive=1)
m1=VKMLgui.Menu(title='Menu',parser=p)
VKMLgui.Parameter(name='min_Y',menu=m1,variable=int,default=300,interval=(0,4000))
VKMLgui.Parameter(name='max_Y',menu=m1,variable=int,default=2000,interval=(0,4000))
VKMLgui.Parameter(name='omega_1',menu=m1,variable=int,default=15000,interval=(-30000,30000))
VKMLgui.Parameter(name='omega_2',menu=m1,variable=int,default=12000,interval=(-30000,30000))
VKMLgui.Parameter(name='temperature',menu=m1,variable=int,default=1300,interval=(0,2000))
p.add_command(printPhase)
p.add_command(printCommonTangent)
p.run()
