# -*- coding: utf-8 -*-

#
#  guiPhaseSolverC.py
#  Swig_Test
#
#  Created by Thomas on 9/25/09.
#  Copyright (c) 2009. All rights reserved.
#
# example:  Example of more complicated example of what is possible using the matplotlib viewer
#           Uses Matplotlib and the C solvers
#           draws free energy curves at two seperate temperatures and a full phase diagram in one image

from gibbs.tools import VKMLgui
from gibbs.variables import RegularFreeEnergyVariable
from gibbs.solvers.qhsolver.qhbinarysolver import CommonTangentSolver, PhaseDiagramSolver
#from gibbs.viewers import GnuplotViewer
from gibbs.viewers.matplotlib import MatplotlibViewer

Viewer=MatplotlibViewer(title='More Complex Example',colorList=['b','g'])
def callback(Omega_1,Temperature_1,Temperature_2):

    DG1=RegularFreeEnergyVariable(omega=Omega_1,delta_Ha=0,delta_Hb=7280,Tma=4000,Tmb=1000)
    DG2=RegularFreeEnergyVariable(omega=-22880,delta_Ha=11940,delta_Hb=24263,Tma=1329,Tmb=2135)
    DG=[DG1,DG2]

    #solvers for the two free energies
    TangentSolver1=CommonTangentSolver(DG=DG,
               x=[0,1,200],
               y=[Temperature_1,1300,200])
               
    TangentSolver2=CommonTangentSolver(DG=DG,
               x=[0,1,200],
               y=[Temperature_2,1300,200])
    
    #display the Tangents
    Viewer.plot(vars=TangentSolver1,
                    subplot='221',                              #matlab/matplotlib style subfigure
                    xlabel='C',                                 #x-label
                    ylabel='$\\Delta(G)$',                      #y-label with matplotlib style tex
                    title=('subplot 1: T=%i'%(Temperature_1)))  #title
    #same as previous
    Viewer.plot(vars=TangentSolver2,subplot='222',xlabel='C',ylabel='$\\Delta(G)$',title=('subplot 2: T=%i'%(Temperature_2)))
    
#Callback for the phase diagram, only called if Omega value changes
def phaseCallback(Omega_1):
    DG1=RegularFreeEnergyVariable(omega=Omega_1,delta_Ha=0,delta_Hb=7280,Tma=4000,Tmb=1000)
    DG2=RegularFreeEnergyVariable(omega=-22880,delta_Ha=11940,delta_Hb=24263,Tma=1329,Tmb=2135)
    
    #Solver for Phase Diagrams
    PhaseSolver=PhaseDiagramSolver(DG=[DG1,DG2],    #FreeEnergyVariables
                                x=[0,1,200],        #x-range
                                y=[600,4000,350])   #y-range
    Viewer.plot(vars=PhaseSolver,
                subplot='212',      #matlab/matplotlib style subfigure
                xlabel='C',         #xlabel
                ylabel='Temp',      #ylabel
                title='subplot 3')  #title for subplot

def saveImage(**kwargs):
    l.save(file=kwargs['file'])
	
    
if __name__=='__main__':

    p=VKMLgui.Parser(title='QHSolver')
    m1=VKMLgui.Menu(title='x Properties',parser=p)
    VKMLgui.Parameter(name='Temperature_1',menu=m1,variable=int,default = 1500, interval=(0,4000))
    VKMLgui.Parameter(name='Temperature_2',menu=m1,variable=int,default = 1700, interval=(0,4000))
    VKMLgui.Parameter(name='Omega_1',menu=m1,variable=int,default=0,interval=(-30000,30000))
    VKMLgui.Parameter(name='file',menu=m1,variable=str,default='Image.eps')
    VKMLgui.Parameter(name='save',menu=m1,variable='function',default=saveImage)


    p.add_command(callback)
    p.add_command(phaseCallback)



    p.run()
