/*
 *  CHSolver.h
 *  Swig_Test
 *
 *  Created by Thomas on 9/14/09.
 *  Copyright 2009 __MyCompanyName__. All rights reserved.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/timeb.h>
#include <qhull/qhull_a.h>
#include <qhull/qset.h>
#include <qhull/qhull.h>

struct qhull_tangent_t
{
	int nb_tangents;
	double Temp;
	double *xx; //2 x and 2y per tangent
	double *yy;
    double *phaseNumber;
    //double *xhull;
    //double *yhull;
    //int count;
};
typedef struct qhull_tangent_t qhull_tangent_t;

/*struct array_struct
{
    int nbelements;
    double *array;
}
typedef struct array_struct array_struct;*/

struct mat_properties_t
{
	double omega;
	double delta_Ha;
	double delta_Hb;
	double temp_m_a;
	double temp_m_b;
};
typedef struct mat_properties_t mat_properties_t;
extern double getDouble (double *a,int b);
extern void	setDouble (double *a,int index, double value);
extern void freeStruct(qhull_tangent_t structs);
extern void freeArray(double *arrays);

void releaseDoubleArray(double *a);

void printa(char *s);

extern void printToFile(double *points, double *freeEnergy, int numpoints);

extern mat_properties_t setMatStruct(double omega, double delta_Ha, double delta_Hb, double temp_m_a, double temp_m_b);

/***********************************
 / calculates the free energies for certain materials and puts them in the proper free energy array
 / points				array containing all the free energies for the materials		(see: getPointsArray)
 / freeEnergy			array to fill initialized with same amount of elements as points
 / nb_pts				is the total number of data points per material (ex: 200)
 / elt_number			number of the material (should not be bigger than nb_elements in getPointsArray
 / totalmax				is number of points for which points is malloc'ed.				(see: getPointsArray)
 /**********************************/
extern int calculateFreeEnergyArrays(double *points, double *freeEnergy, int totalmax, int nb_pts, int elt_numbers, mat_properties_t material, double Temp );


/**************************
 / nb_pts				is the total number of data points per material (ex: 200)
 / nb_elements			is number of different materials (ex: 3)
 / totalmax				is number of points for which points is malloc'ed. It should be equal to nb_pts*nb_elements. 
 /						If it is bigger, it does not matter, if it is smaller, it will return a kOVERFLOWcatch error (1001)
 / points				is a double array alloc'ed with totalmax elements
 /*************************/
extern int getPointsArray(double *points, int nb_pts, int nb_elements, int totalmax, double min, double max);


/**************************
 /function returning a qhull_tangent_t structure of all the tangent points Extremities
 ///return:
 /nb_tangents		integer representing the number of tangents found at certain temp
 /xx				array with all the x points for the tangents (2 points per tangent): if 3 tangents, 6 xx points
 /yy				array with all the y points for the tangents (2 points per tangent)
 /Temp				temperature at which the tangent was computed
 /
 //inputs
 /T					temperature
 /numpoints			number of elements * number of points per element
 /points			points x-coordinates: (see getPointsArray)
 /freeEnergy		freeEnergies corresponding to points Array (see calculateFreeEnergiesArray)
 **************************/
extern qhull_tangent_t TCQHSolver(double T, int numpoints,int variables, double *points,double *freeEnergy);
extern qhull_tangent_t QHSolver(double T, int numpoints,int variables, double *points,double *freeEnergy);


extern int cube( int n );
