# -*- coding: utf-8 -*-
#
#  CHCaller.py
#  Swig_Test
#
#  Created by Thomas on 9/14/09.
#  Copyright (c) 2009 __MyCompanyName__. All rights reserved.
from Variable import Variable
from pylab import *
from sympy import *
from QHSympyFunctions import *
import QHSolve
#import threading
import time

###################
# Future work: Threading to speed up Python Generation
###################

#class computeThread (threading.Thread):
#    def __init__(self, DG,points,Temp, material,pointsPerPhase,freeEnergy):
#        self.DG=DG
#        self.Temp=Temp
#        self.material=material
#        self.pointsPerPhase=pointsPerPhase
#        self.freeEnergy=freeEnergy
#        self.points=points
#        #print self.points
#        threading.Thread.__init__(self)
#    def run(self):
#        DGVariable=Variable(DG)
#        material=self.material
#        a=DGVariable(omega=material.omega,R=8.314,Temp=self.Temp,delta_Ha=self.material.delta_Ha,delta_Hb=self.material.delta_Hb,Tmb=self.material.temp_m_b,Tma=self.material.temp_m_a)
#        #print a
#        ll=Variable(a)
#        for j in range(self.pointsPerPhase):
#            self.freeEnergy[j]=ll(x=self.points[j]).evalf()
#        #print self.freeEnergy
#
#
#def computePythonMultiThreadingDef(DG,points,Temp,material,freeEnergy,**kwargs):
#        DGVariable=Variable(DG)
#        a=DGVariable(omega=material.omega,R=8.314,Temp=Temp,delta_Ha=material.delta_Ha,delta_Hb=material.delta_Hb,Tma=material.temp_m_a,Tmb=material.temp_m_b)
#        ll=Variable(a)
#        for j in range(self.pointsPerPhase):               
#            freeEnergy[j]=float(ll(x=points[i*self.pointsPerPhase+j]).evalf())

        
        
        
class QHSolver():
    '''
    QHSolver Class
    solves the Convex Hull based on different options
    
    inputs:
    pointsPerPhase      Number of points on x-axis
    phaseCalucations    Number of different Temperatures Computed
    maxPhaseTemp        Maximum Temperature computed
    minPhaseTemp        Minimum Temperature computed
    
    additional kwargs
    DGList              List of different Sympy expressions representing 
                        free energies of different phases, should only have
                        Symbol x left
    
    Not Working Yet
    FEClass             free Energy computation class (usually SWIG)
    FEParameters        list of structures to pass to the FEClass
    '''
                        
    def __init__(self,pointsPerPhase=200,phaseCalculations=150,maxPhaseTemp=1300,minPhaseTemp=650,**kwargs):
        self.pointsPerPhase=pointsPerPhase
        self.phaseCalculations=phaseCalculations
        self.minPhaseTemp=minPhaseTemp
        self.maxPhaseTemp=maxPhaseTemp
        self.MaterialsList=[]
        self.MaterialsListExtras=[]
       	self.P=[]
        self.L=[]
        if kwargs.has_key('DGList'):
            self.DGList=kwargs['DGList']
            self.computeMethod='sympy'
        elif kwargs.has_key('FEClass') and kwargs.has_key('FEList'):
            self.computeMethod='fe'
            self.FEList=kwargs['FEList']
            self.FEGenerator=kwargs['FEClass']
        else:
            self.computeMethod='semiRegularC'
            
            
        #SaveFileOptions    
        if kwargs.has_key('savePath'):
            self.savePath=kwargs['savePath']
        else:
            self.savePath=''
        if kwargs.has_key('saveName'):
            self.saveName=kwargs['saveName']
        else:
            self.saveName=None
        
        self.autorun=0
        #autorun: should it start computing at once (default yes)    
        if kwargs.has_key('autorun'):
            self.autorun=kwargs['autorun']
        else:
            self.autorun=1
        
        #autorun with sympy
        if (self.computeMethod=='sympy') and  (self.autorun is not 0):
            self.solvePhase()
	
	############################################
	# kwargs are a dictionary of extra properties
	# they cannot be used to generate the free energies under C
	# they are generated using Python
	# and thus will be much much slower
	# They Give much more variability
	############################################
    def addMaterial(self,omega,delta_Ha,delta_Hb,Tma,Tmb,**kwargs):
        self.MaterialsList.append(QHSolve.setMatStruct(omega,delta_Ha,delta_Hb,Tma,Tmb))
        self.MaterialsListExtras.append(kwargs)
		
		
	############################################
	# prints the Materials and the Extra properties for the materials
	# the basic properties are stored in a C-SWIG structure 
	# the extra ones are stores in a Python Dict
	#############################################
    def printMaterialsList(self):
        print self.MaterialsList
        print self.MaterialsListExtras
        
        
	#############################################
    #calculate the phase points using the builtin 
    #C function (very fast)
    #
    #############################################	
    def _calulatePhasePoints(self,points,Temp):
        freeEnergy=QHSolve.doubleArray(self.pointsPerPhase*len(self.MaterialsList))

       	for i in range(len(self.MaterialsList)):
            QHSolve.calculateFreeEnergyArrays(points,freeEnergy,self.pointsPerPhase*len(self.MaterialsList),self.pointsPerPhase, i,self.MaterialsList[i],Temp)
        self.freeEnergy=freeEnergy
       	return freeEnergy
		
	#############################################
    # calculate free Energy using sympy 
    # very very slow
    # very flexible
    #############################################
    def _computeSympy(self,points,Temp,**kwargs):
        #freeEnergy=[0.0]*(self.pointsPerPhase*len(self.MaterialsList))#
        freeEnergy=QHSolve.doubleArray(self.pointsPerPhase*len(self.DGList))
        for i in range(len(self.DGList)):
            DGVariable=Variable(self.DGList[i])
            for j in range(self.pointsPerPhase):  
                print DGVariable(x=points[i*self.pointsPerPhase+j],Temp=Temp).evalf()
                freeEnergy[i*self.pointsPerPhase+j]=float(DGVariable(x=points[i*self.pointsPerPhase+j],Temp=Temp).evalf())
        return freeEnergy
        
    #############################################
    # calculate free Energy using extendable modules 
    # can be either slow or fast depending on the module type
    # C/SWIG very fast
    # python slow
    # and so forth
    #############################################
    def _calulateFE(self,points,Temp):
        freeEnergy=QHSolve.doubleArray(self.pointsPerPhase*len(self.FEParameters))
        for i in range(len(self.FEParameters)):
            self.FEGenerator.calculateFreeEnergyArrays(points,freeEnergy,self.pointsPerPhase*len(self.FEParameters),self.pointsPerPhase, i,self.FEParameters[i],Temp)
        return freeEnergy
            
    
###################
# Future work: Threading to speed up Python Generation
###################
    
#    def _computePythonMultiThreading(self,DG,points,Temp,materialNumber,freeEnergy,**kwargs):
#        material=self.MaterialsList[materialNumber]
#        DGVariable=Variable(DG)
#        a=DGVariable(omega=material.omega,R=8.314,Temp=Temp,delta_Ha=material.delta_Ha,delta_Hb=material.delta_Hb,Tma=material.temp_m_a,Tmb=material.temp_m_b)
#        ll=Variable(a)
#        for j in range(self.pointsPerPhase):               
#            freeEnergy[a]=(float(ll(x=points[i*self.pointsPerPhase+j]).evalf()))
#        return freeEnergy
#        
#    def _multithreadRun(self,DG,points,Temp,**kwargs):
#        #apoints=self._getPoints()
#        
#        aaa=[0]*self.pointsPerPhase
#        bbb=[0]*self.pointsPerPhase
#        ccc=[0]*self.pointsPerPhase
#        
#        material1=self.MaterialsList[0]
#        thread1 = computeThread(DG=DG,points=points,Temp=Temp, material=material1, pointsPerPhase=self.pointsPerPhase,freeEnergy=aaa)
#        thread1.start()
#        
#        
#        material2=self.MaterialsList[1]
#        thread2 = computeThread(DG=DG,points=points,Temp=Temp, material=material2, pointsPerPhase=self.pointsPerPhase,freeEnergy=bbb)
#        thread2.start()
#
#        #material3=self.MaterialsList[2]
#        #thread3 = computeThread(DG=DG,points=points,Temp=Temp, material=material3, pointsPerPhase=self.pointsPerPhase,freeEnergy=ccc)
#        #thread3.start()
#
#        thread1.join()
#        thread2.join()
#        #thread3.join()
#
#        
#           # thread.start_new_thread( print_time, (DG,points,Temp,0,ccc) )
#           # thread.start_new_thread( print_time, (DG,points,Temp,1,aaa ) )
#           # thread.start_new_thread( print_time, (DG,points,Temp,2,bbb ) )
		
			
    #############################################
    # generates 'x' data points  
    # returns a swig object pointer to C double
    # array
    #############################################
    def _getPoints(self,phases):
       	points		= QHSolve.doubleArray(self.pointsPerPhase*phases)
       	errorVal	= QHSolve.getPointsArray(points,self.pointsPerPhase,phases,self.pointsPerPhase*phases)
       	return points
        
###################
# Future work: Threading to speed up Python Generation
###################
		
#    def solveMT(self,DG):
#        t1=time.time()
#        points=self._getPoints()
#        #self.points=self._getPoints()
#        for i in range(self.phaseCalculations):
#            self._multithreadRun(DG=DG,points=points,Temp=700)
#        t2=time.time()
#        
#        
#        for i in range(self.phaseCalculations):
#            self._computePython(DG=DG,points=points,Temp=700)
#        t3=time.time()
#        print 'time: '+str(t2-t1)
#        print 'time: '+str(t3-t2)
		
    def solvePhase(self, **kwargs):
        
        if self.computeMethod=='sympy':
            phases=len(self.DGList)                                                     #number of phases from DGList
        elif self.computeMethod=='semiRegularC':
            phases=len(self.MaterialsList)                                              #number of phases from MaterialsList
        elif self.computeMethod=='FEClass':
            phases=len(self.FEParameters)                                               #number of phases from FEParameterList
        
        points = self._getPoints(phases)                                                #get points array
            
        totalmax=self.pointsPerPhase*phases                                             #total number of free energies for Qhull
        
        stepsize=(int)(self.maxPhaseTemp-self.minPhaseTemp)/self.phaseCalculations      #stepsize for temperature increase
        
        
        for Temp in range(self.minPhaseTemp,self.maxPhaseTemp,stepsize):                #main loop
            print Temp
            if self.computeMethod=='sympy':
                freeEnergy=self._computeSympy(points,Temp)                              #compute from sympy equation saved previously
            elif self.computeMethod=='semiRegularC':
                freeEnergy=self._calulatePhasePoints(points,Temp)                       #compute from built in C funtion
            elif self.computeMethod=='FEClass':
                freeEnergy=self._calulateFE(points,Temp)                                #compute from external Free Energy package
            
            c=QHSolve.TCQHSolver(Temp,totalmax,phases,points,freeEnergy)                #run the convex hull solver itself

            for j in range(c.nb_tangents):                                              #add points to Lists
                    self.P.append(QHSolve.getDouble(c.xx,(j*2)))
                    self.P.append(QHSolve.getDouble(c.xx,(j*2 + 1)))
                    self.L.append(c.Temp)
                    self.L.append(c.Temp)

    def printPL(self):
        for i in range(len(self.P)):
            print str(self.P[i]) +'	'+str(self.L[i])
			
    def printPhaseDiagram(self, **kwargs):
		#for i in range(len(self.P)):
		#	plot([self.P[i]],[self.L[i]],marker='o',markersize=2,color='b')
        fig2=figure()
        ax=fig2.add_subplot(111)
        ax.axis([0,1,self.minPhaseTemp,self.maxPhaseTemp])
		#ax=axes(0,1,self.minPhaseTemp,self.maxPhaseTemp)
        ax.scatter(self.P,self.L,marker='o',color='k')
        ax.set_xlim(0,1)
        ax.set_ylabel('T')
        
        #setting the title
        title=''
        if(kwargs.has_key('Title')):
            title=kwargs['Title']
        else:
            title='Phase Diagram'
        ax.set_title(title)
        #grid()
        
        if(kwargs.has_key('save')):
            if(kwargs.has_key('savePath')):
                self.savePath=kwargs['savePath']
            if(kwargs.has_key('saveName')):
                self.saveName=kwargs['saveName']
            if(self.saveName==None):
                self.saveName=title
            savePath=self.savePath+self.saveName+'.ps'
            fig2.savefig(savePath)
        else:
            grid()
            show()
        close()


    


		
		
		

