//
//    QHSolve.c
//    gibbs
//
//
//    Created by Thomas Cool on 1-Sept-2009.
//    Copyright 2009 Thomas Cool, R. Edwin Garcia
//    This file is part of gibbs.
//
//    gibbs is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

#include "QHSolve.h"
#include <sys/timeb.h>
#define kOVERFLOWcatch	1001
#define kErrorOne		1


double max_array(double a[], int num_elements)
{
	int i;
	double max=a[0];
	for (i=1; i<num_elements; i++)
	{
		if (a[i]>max)
		{
			max=a[i];
		}
	}
	return(max);
}

int cube( int n )
{
  return n * n * n;
}
int calc(double * points,int len)
{
    int i;
    for(i=0;i<len;i++)
    {
        points[i]=i;
    }
    return 0;
}
double xlogx (double a)
{
	
	if  (a==0.0)
	{//printf("!!");
	return 0;}
	else if (a == 1.0)
	{	//printf("@@");
	return 0;}
	else if( a>0 && a<1)
		//printf(" value %f %f\n",a, log(a));
		return a*log(a);
	return 0.0;
}
double getDouble (double *a,int b)
{
	return a[b];
}
void releaseDoubleArray(double *a)
{
    free(a);
}
void setDouble(double *a, int index, double value)
{
	a[index]=value;
	//return 0;
}

void setDoubleInSA(array_struct_t s, int loc, double value)
{
    if (loc < s.nbelements)
    {
        s.array[loc]=value;
    }
}

double getDoubleInSA(array_struct_t s, int loc)
{
    if (loc < s.nbelements)
    {
        return s.array[loc];
    }
    return 0.0;
}

array_struct_t createSArray(int nbelements)
{
    array_struct_t s;
    s.nbelements = nbelements;
    s.array = (double*) malloc(nbelements * sizeof(double));
    int i;
    for (i=0;i<s.nbelements;i++)
    {
        s.array[i]=0.0;
    }
    return s;
}

void printToFile(double *points, double *freeEnergy, int numpoints)
{	
	FILE *fp;
	
	printf("This will display on the screen.\n");
	
	if((fp=freopen("OUT2.txt", "w" ,stdout))==NULL) {
		printf("Cannot open file.\n");
	}
	int ii;
	for (ii=0;ii<numpoints;ii++)
	{
		printf("%lf	%lf\n", points[ii],freeEnergy[ii]);
	}
	
	fclose(fp);
	
}
/********
 / reusable materials properties structure
 ********/

mat_properties_t setMatStruct(double omega, double delta_Ha, double delta_Hb, double temp_m_a, double temp_m_b)
{
	mat_properties_t b;
	b.delta_Ha=delta_Ha;
	b.delta_Hb=delta_Hb;
	b.temp_m_a=temp_m_a;
	b.temp_m_b=temp_m_b;
	b.omega=omega;
	return b;
}


/**************************
 / nb_pts				is the total number of data points per material (ex: 200)
 / nb_elements			is number of different materials (ex: 3)
 / totalmax				is number of points for which points is malloc'ed. It should be equal to nb_pts*nb_elements. 
 /						If it is bigger, it does not matter, if it is smaller, it will return a kOVERFLOWcatch error (1001)
 / points				is a double array alloc'ed with totalmax elements
 /*************************/
int getPointsArrayN(double *points, int len, int nb_elements, int nb_pts, double min, double max)
{
	if(nb_pts*nb_elements>len)
		return kOVERFLOWcatch;
	int i;
	points[0]=min;
	for(i=1;i<nb_pts;i++)
	{
		points[i]=points[i-1]+max/(double)(nb_pts-1);
	}
	for(i=nb_pts;i<(nb_pts*nb_elements);i++)
	{
		points[i]=points[i-nb_pts];
	}
	return 0;
}

int getPointsArray(double *points, int nb_pts, int nb_elements, int totalmax, double min, double max)
{
	if(nb_pts*nb_elements>totalmax)
		return kOVERFLOWcatch;
	int i;
	points[0]=min;
	for(i=1;i<nb_pts;i++)
	{
		points[i]=points[i-1]+max/(double)(nb_pts-1);
	}
	for(i=nb_pts;i<(nb_pts*nb_elements);i++)
	{
		points[i]=points[i-nb_pts];
	}
	return 0;
}
array_struct_t getPointsArraySA( int nb_pts, int nb_elements,int totalmax, double min, double max)
{
    array_struct_t s;
    s.nbelements=nb_elements*nb_pts;
    s.array = (double*) malloc(nb_elements * sizeof(double));

    int i;
    s.array[0]=min;
	for(i=1;i<nb_pts;i++)
	{
		s.array[i]=s.array[i-1]+max/(double)(nb_pts-1);
	}
	for(i=nb_pts;i<(nb_pts*nb_elements);i++)
	{
		s.array[i]=s.array[i-nb_pts];
	}
    return s;
}
/*extern (double *)generatePointsArray( int nb_pts, int nb_elements,int totalmax, double min, double max)
{
    double *array;
    s.nbelements=nb_elements*nb_pts;
    array = (double*) malloc(nb_elements * sizeof(double));
    
    int i;
    array[0]=min;
	for(i=1;i<nb_pts;i++)
	{
		array[i]=array[i-1]+max/(double)(nb_pts-1);
	}
	for(i=nb_pts;i<(nb_pts*nb_elements);i++)
	{
		array[i]=array[i-nb_pts];
	}
    return array;
}*/

/***********************************
 / calculates the free energies for certain materials and puts them in the proper free energy array
 / points				array containing all the free energies for the materials		(see: getPointsArray)
 / freeEnergy			array to fill initialized with same amount of elements as points
 / nb_pts				is the total number of data points per material (ex: 200)
 / elt_number			number of the material (should not be bigger than nb_elements in getPointsArray
 / totalmax				is number of points for which points is malloc'ed.				(see: getPointsArray)
 **********************************/
 
 
int calculateFreeEnergyArrays(double *points, double *freeEnergy, int totalmax, int nb_pts, int elt_number, mat_properties_t material, double Temp )
{
	double R=8.314;
	int i;
	for(i=0;i<nb_pts;i++)
	{
		int index = i + elt_number*nb_pts;
		if(i==(nb_pts-1))
		{
			freeEnergy[index]= material.delta_Hb-(Temp*material.delta_Hb/material.temp_m_b);
		}
		else
		{
			freeEnergy[index] = material.omega * points[index] *(1.0-points[index])+R*Temp* (xlogx(points[index])+xlogx(1.0-points[index]) )+ (1.0-points[index])*(material.delta_Ha - Temp*material.delta_Ha/material.temp_m_a)+points[index]*(material.delta_Hb-Temp*material.delta_Hb/material.temp_m_b);
            //printf("%lf %lf\n",points[index],freeEnergy[index]);
		}
	}
	return 0;
}
/**************************
 /function returning a qhull_tangent_t structure of all the tangent points Extremities
 ///return:
 /nb_tangents		integer representing the number of tangents found at certain temp
 /xx				array with all the x points for the tangents (2 points per tangent): if 3 tangents, 6 xx points
 /yy				array with all the y points for the tangents (2 points per tangent)
 /Temp				temperature at which the tangent was computed
 /
 //inputs
 /T					temperature
 /numpoints			number of elements * number of points per element
 /points			points x-coordinates: (see getPointsArray)
 /freeEnergy		freeEnergies corresponding to points Array (see calculateFreeEnergiesArray)
 **************************/

qhull_tangent_t TCQHSolver(double T, int numpoints,int variables, double *points,double *freeEnergy)
{
	//setting up data for qhull function

	int pointsPerPhase=(int)(numpoints/variables);
	int dim=2;
	int i;
	coordT *qpoints;
	facetT *facet;
	//Alloc qpoints array 
	if (!(qpoints = (double*)malloc ((dim*numpoints)*sizeof(coordT)))) // allocates memory for an array that will copy a point(concentration) in an index and their corresponding free energy in the next index number.
	{ 
		printf ("qhullCalcHull: could not allocate memory for points\n");
		exit(1) ; 
	}
	for (i=0; i<numpoints; i++)
	{
		qpoints[dim*i]   = (coordT) points[i];
		qpoints[dim*i+1] = (coordT) freeEnergy[i];
	}
	
	//calling qhull
	qh_init_A (stdin, stdout, stderr, 0, NULL);
	qh_init_B (qpoints, numpoints, dim, False);
	qh_qhull (); 
	
	free(qpoints);
	//qh_printsummary (stdout);
	
	//get data from qhull
    
	int  qhull_array[qh num_facets * dim];
//qhull_array = (int *)malloc((qh num_facets * dim)*sizeof(int));
	
	int j=0,count=0;
	FORALLfacets
    {    
		vertexT *vertex, **vertexp;
		FOREACHvertex_(facet->vertices)
		{
			int thepoint = (vertex->point-qh first_point)/2.0;
			if(thepoint<numpoints);
				qhull_array [count++] = thepoint;
		}
    }
	int k;
	for(k=0;k<count;k++)
	{
		for (j=k+1;j<count;j++)
		{
			if ( points[qhull_array[k]] > points[qhull_array[j]] )
			{
				int temp = qhull_array[k];
				qhull_array[k] = qhull_array[j];
				qhull_array[j] = temp;
			}
		}
	}
    //printa('aaa');
	//Parsing Data from Qhull
	qhull_tangent_t eq;
	eq.Temp=T;
	eq.nb_tangents=0;
	double gap = (points[2]-points[1])*1.1;
    int *temparray;// = new temparray[count];

    //printa('bbb');
    temparray=(int *)malloc(count*sizeof(int));
    int tempcounter;
    //eq.xhull = (double*) malloc(count * sizeof(double));
    //eq.yhull = (double*) malloc(count * sizeof(double));
    tempcounter=0;

    int phases;
    for (i=0;i<count;i++)
    {
        phases=0;
        if(qhull_array[i-1]>=pointsPerPhase)
        {
            phases=(int)qhull_array[i-1]/pointsPerPhase;
        }
        int pts=qhull_array[i-1]-phases*pointsPerPhase;
        int addones=1;
        double yy_ones = freeEnergy[qhull_array[i]];
        for (j=0;j<variables;j++)
        {
            //printf("j: %i\n",j);
            if(freeEnergy[j*pointsPerPhase+pts]<yy_ones)
            {
                addones=0;
                
            }
        }
        if (addones==1)
        {
            temparray[tempcounter]=qhull_array[i];
            tempcounter++;
            //eq.xhull[tempcounter]=points[qhull_array[i]];
            //eq.yhull[tempcounter]=freeEnergy[qhull_array[i]];
        }
        
        
    }
   
	for (i=1;i<tempcounter;i++)
	{
		
		if(points[temparray[i]]-points[temparray[i-1]]>gap)
		{
			eq.nb_tangents++;			//printf("row: %i, point : %lf\n",i, points[qhull_array[i]] );
            //printf("gap from %lf to %lf\n",points[qhull_array[i-1]],points[qhull_array[i]]);
		}
        
	}

    //printf("nb tangents: %i Temp:%i\n",eq.nb_tangents,eq.Temp);
	eq.xx = (double*) malloc(eq.nb_tangents*2 * sizeof(double));
	eq.yy = (double*) malloc(eq.nb_tangents*2 * sizeof(double));
    //eq.xhull = (double*) malloc(count * sizeof(double));
    //eq.yhull = (double*) malloc(count * sizeof(double));
    eq.phaseNumber = (double*) malloc(tempcounter * sizeof(double));
	int tangent = 0;
    //eq.xhull[0]=points[qhull_array[0]];
    //eq.xhull[1]=points[qhull_array[1]];
    //eq.yhull[0]=freeEnergy[qhull_array[0]];
    //eq.yhull[1]=freeEnergy[qhull_array[1]];
    
//	for (i=2;i<count;i+=1)
//	{
//		//eq.xhull[i]=points[qhull_array[i]];
//        //eq.yhull[i]=freeEnergy[qhull_array[i]];
//		if(points[qhull_array[i]]-points[qhull_array[i-1]]>gap)
//		{
//            int phase;
//            if(qhull_array[i-1]<pointsPerPhase)
//            {
//                phase=0;
//            }
//            else
//            {
//                phase=(int)qhull_array[i-1]/pointsPerPhase;
//            }
//            int phase2;
//            if(qhull_array[i]<pointsPerPhase)
//            {
//                phase2=0;
//            }
//            else
//            {
//                phase2=(int)qhull_array[i]/pointsPerPhase;
//            }
//            int pt=qhull_array[i-1]-phase*pointsPerPhase;
//            int pt2=qhull_array[i]-phase2*pointsPerPhase;
//            //printf("qhull_pt: %i phase: %i, point:%i, pointsPerPhase: %i\n",qhull_array[i-1],phase,pt,pointsPerPhase);
//            //printf("qhull_pt2: %i phase2: %i, point2:%i, pointsPerPhase: %i\n",qhull_array[i],phase2,pt2,pointsPerPhase);
//
//
//            double yy_one = freeEnergy[qhull_array[i-1]];
//            double yy_two = freeEnergy[qhull_array[i]];
//            int addone=1;
//            int addtwo=1;
//            double addVal1=0.0;
//            double addVal2=0.0;
//            int phaseOne=100;
//            int phaseTwo=100;
//            for (j=0;j<variables;j++)
//            {
//                //printf("j: %i\n",j);
//                if(freeEnergy[j*pointsPerPhase+pt]<yy_one)
//                {
//                    //addone=0;
//                    //printf("phase: %i, qhulled: %lf, calc: %lf\n",j,yy_one,freeEnergy[j*pointsPerPhase+pt]);
//                    addone=2;
//                    addVal1=freeEnergy[j*pointsPerPhase+pt];
//                    
//                }
//                else if (freeEnergy[j*pointsPerPhase+pt]==yy_one)
//                {
//                    phaseOne=j;
//                }
//                if(freeEnergy[j*pointsPerPhase+pt2]<yy_two)
//                {
//                    //addtwo=0;
//                    //printf("phase: %i, qhulled: %lf, calc: %lf\n",j,yy_two,freeEnergy[j*pointsPerPhase+pt2]);
//                    addtwo=2;
//                    addVal2=freeEnergy[j*pointsPerPhase+pt2];
//                    //qhull_array[i]=j*pointsPerPhase+pt2;
//                }
//                else if (freeEnergy[j*pointsPerPhase+pt2]==yy_two)
//                {
//                    phaseTwo=j;
//                }
//                           
//            }
//            if(addone==1)// || addone==2)
//            {
//                eq.xx[tangent]=	points[qhull_array[i-1]];
//                if (addone==3)
//                {
//                    eq.yy[tangent]= addVal1;
//                }else
//                    eq.yy[tangent]= freeEnergy[qhull_array[i-1]];
//                
//                eq.phaseNumber[tangent]=phaseOne;
//                tangent++;
//            }
//            if(addtwo==1)// ||addtwo==2)
//            {
//                eq.xx[tangent]= points[qhull_array[i]]; 
//                if (addtwo==3)
//                {
//                    eq.yy[tangent]= addVal2;
//                }else
//                    eq.yy[tangent]= freeEnergy[qhull_array[i]];
//                
//                //yy[tangent]= freeEnergy[qhull_array[i]];
//                eq.phaseNumber[tangent]=phaseTwo;
//                tangent++;
//            }
    //printf("in loop\n");
	for (i=1;i<tempcounter;i+=1)
	{
		//eq.xhull[i]=points[temparray[i]];
        //eq.yhull[i]=freeEnergy[temparray[i]];
		if(points[temparray[i]]-points[temparray[i-1]]>gap)
		{
            int phase;
            if(temparray[i-1]<pointsPerPhase)
            {
                phase=0;
            }
            else
            {
                phase=(int)temparray[i-1]/pointsPerPhase;
            }
            int phase2;
            if(temparray[i]<pointsPerPhase)
            {
                phase2=0;
            }
            else
            {
                phase2=(int)temparray[i]/pointsPerPhase;
            }
            //int pt=temparray[i-1]-phase*pointsPerPhase;
            //int pt2=temparray[i]-phase2*pointsPerPhase;
            //printf("qhull_pt: %i phase: %i, point:%i, pointsPerPhase: %i\n",temparray[i-1],phase,pt,pointsPerPhase);
            //printf("qhull_pt2: %i phase2: %i, point2:%i, pointsPerPhase: %i\n",temparray[i],phase2,pt2,pointsPerPhase);
            
            
            //double yy_one = freeEnergy[temparray[i-1]];
            //double yy_two = freeEnergy[temparray[i]];
            int addone=1;
            int addtwo=1;
            double addVal1=0.0;
            double addVal2=0.0;
//            int phaseOne=100;
//            int phaseTwo=100;
//            for (j=0;j<variables;j++)
//            {
//                //printf("j: %i\n",j);
//                if(freeEnergy[j*pointsPerPhase+pt]<yy_one)
//                {
//                    //addone=0;
//                    //printf("phase: %i, qhulled: %lf, calc: %lf\n",j,yy_one,freeEnergy[j*pointsPerPhase+pt]);
//                    addone=2;
//                    addVal1=freeEnergy[j*pointsPerPhase+pt];
//                    
//                }
//                else if (freeEnergy[j*pointsPerPhase+pt]==yy_one)
//                {
//                    phaseOne=j;
//                }
//                if(freeEnergy[j*pointsPerPhase+pt2]<yy_two)
//                {
//                    //addtwo=0;
//                    //printf("phase: %i, qhulled: %lf, calc: %lf\n",j,yy_two,freeEnergy[j*pointsPerPhase+pt2]);
//                    addtwo=2;
//                    addVal2=freeEnergy[j*pointsPerPhase+pt2];
//                    //temparray[i]=j*pointsPerPhase+pt2;
//                }
//                else if (freeEnergy[j*pointsPerPhase+pt2]==yy_two)
//                {
//                    phaseTwo=j;
//                }
//                
//            }
            if(addone==1)// || addone==2)
            {
                eq.xx[tangent]=	points[temparray[i-1]];
                if (addone==3)
                {
                    eq.yy[tangent]= addVal1;
                }else
                    eq.yy[tangent]= freeEnergy[temparray[i-1]];
                
                eq.phaseNumber[tangent]=1.0;
                tangent++;
            }
            if(addtwo==1)// ||addtwo==2)
            {
                eq.xx[tangent]= points[temparray[i]]; 
                if (addtwo==3)
                {
                    eq.yy[tangent]= addVal2;
                }else
                    eq.yy[tangent]= freeEnergy[temparray[i]];
                
                //yy[tangent]= freeEnergy[temparray[i]];
                eq.phaseNumber[tangent]=2.0;
                tangent++;
            }
            

            /*if(add==1 || add==0)
            {
                eq.xx[tangent]=	points[qhull_array[i-1]];
                eq.xx[tangent+1]= points[qhull_array[i]];
                eq.yy[tangent]= freeEnergy[qhull_array[i-1]];//printf("row: %i, point : %lf\n",i, points[qhull_array[i]] );
                eq.yy[tangent+1]= freeEnergy[qhull_array[i]];
                tangent+=2;
            }
            else
            {
                eq.nb_tangents--;
            }*/

		}
	}
    eq.nb_tangents=(int)tangent;
    //eq.count=tempcounter;
    
    //printf("eq.nb_tangents: %i\n",eq.nb_tangents);
//free(qhull_array);

	return eq;
	
}
void freeStruct(qhull_tangent_t structs)
{
    free(structs.xx);
    free(structs.yy);
    free(structs.phaseNumber);
}
void freeArray(double *arrays)
{
    free(arrays);
}

qhull_tangent_t QHSolver(double T, int numpoints,int variables, double *points,int len,double *freeEnergy,int lenn)
{
    /*setting up data for qhull function 
    / very important needs to be used to clear memory
    */
    int curlong, totlong;
    
    /*
     *Computing number of points in each phase, will need
     *to be done differently for ternary
     */
    
	int pointsPerPhase=(int)(numpoints/variables);
    /*
     *Number of dimensions for qhull
     *Set to two as this is for a binary solver --- does not mean two phases, just two dimensions
     */
	int dim=2;
	int i;
	coordT *qpoints;
	facetT *facet;
	//Alloc qpoints array 
	if (!(qpoints = (double*)malloc ((dim*numpoints)*sizeof(coordT)))) // allocates memory for an array that will copy a point(concentration) in an index and their corresponding free energy in the next index number.
	{ 
		printf ("qhullCalcHull: could not allocate memory for points\n");
		exit(1) ; 
	}
    //creating the qhull input array for two dimensions
	for (i=0; i<numpoints; i++)
	{
		qpoints[dim*i]   = (coordT) points[i];
		qpoints[dim*i+1] = (coordT) freeEnergy[i];
	}
	
	//calling qhull
	qh_init_A (stdin, stdout, stderr, 0, NULL);
	qh_init_B (qpoints, numpoints, dim, False);
	qh_qhull (); 
	//Free qhull input (we are trying to be memory efficient here
	free(qpoints);
    
    /*
     *If you want to see the output for qhull uncomment the following line,
     *Be ready for a lot of output
     */
	//qh_printsummary (stdout);
	
    
	int  qhull_array[qh num_facets * dim];
    /*
     *Extracting the data from the qhull array and formatting it for ease of use
     */
	int j=0,count=0;
	FORALLfacets
    {    
		vertexT *vertex, **vertexp;
		FOREACHvertex_(facet->vertices)
		{
			int thepoint = (vertex->point-qh first_point)/2.0;
			if(thepoint<numpoints);
            qhull_array [count++] = thepoint;
		}
    }
	int k;
    /*
     *Checking for gaps in the qhull return 
     *Gaps represent common tangents
     */
    double gap = (points[1]-points[0])*1.1;
	for(k=0;k<count;k++)
	{
		for (j=k+1;j<count;j++)
		{
			if ( points[qhull_array[k]] > points[qhull_array[j]] )
			{
				int temp = qhull_array[k];
				qhull_array[k] = qhull_array[j];
				qhull_array[j] = temp;
			}
		}
	}
    qhull_tangent_t eq;
    eq.Temp=T;
    eq.nb_tangents=0;
    
    int temparray[count];
    int tempcounter=0;
    int phases;
    int addToArray=1;
    double yy_ones;
    int pts;
    /*
     *Sometimes qhulls return erroneous data
     *We therefore check to make sure all the data is valid
     *we do a loop over all the returned points to check they are in fact minima
     *if they are not, we discard them
     */
    for (i=0;i<count;i++)
    {
        phases=0;
        if(qhull_array[i]>=pointsPerPhase)
        {
            phases=(int)qhull_array[i]/pointsPerPhase;
        }
        
        pts=qhull_array[i]-phases*pointsPerPhase;
        addToArray=1;
        
        yy_ones = freeEnergy[qhull_array[i]];
        for (j=0;j<variables;j++)
        {
            //printf("j: %i\n",j);
            if(freeEnergy[j*pointsPerPhase+pts]<yy_ones)
            {
                addToArray=0;
                
            }
        }
        if (addToArray==1)
        {
            temparray[tempcounter]=qhull_array[i];
            tempcounter++;
            //eq.xhull[tempcounter]=points[qhull_array[i]];
            //eq.yhull[tempcounter]=freeEnergy[qhull_array[i]];
        }
        
        
    }
    /*
     *Remember the 2 important ints at the beginning
     *They are back here
     *Very important two functions
     *without this, you can have over 65KB ram lost per loop
     */
    qh_freeqhull (!qh_ALL);
    qh_memfreeshort (&curlong, &totlong);
    
    /*
     *Last loop to add the data to the return
     */
    for (i=1;i<tempcounter;i++)
	{
		
		if(points[temparray[i]]-points[temparray[i-1]]>gap)
		{
			eq.nb_tangents++;			//printf("row: %i, point : %lf\n",i, points[qhull_array[i]] );
		}
        
	}
    //printf("nb tangents: %i\n",eq.nb_tangents);
    /*eq.xx = (double*) malloc(eq.nb_tangents*2 * sizeof(double));*/
    eq.yy = (double*)malloc ((eq.nb_tangents*2)*sizeof(double));
    eq.xx = (double*)malloc ((eq.nb_tangents*2)*sizeof(double));
    eq.phaseNumber =(double*)malloc ((eq.nb_tangents*2)*sizeof(double));
	//eq.yy = (double*) malloc((eq.nb_tangents*2) * sizeof(double));
    
    //int tangent = 0;
    int tangent=0;
	for (i=1;i<tempcounter;i+=1)
	{
		//eq.xhull[i]=points[temparray[i]];
        //eq.yhull[i]=freeEnergy[temparray[i]];
		if(points[temparray[i]]-points[temparray[i-1]]>gap)
		{
            int phase;
            if(temparray[i-1]<pointsPerPhase)
            {
                phase=0;
            }
            else
            {
                phase=(int)temparray[i-1]/pointsPerPhase;
            }
            int phase2;
            if(temparray[i]<pointsPerPhase)
            {
                phase2=0;
            }
            else
            {
                phase2=(int)temparray[i]/pointsPerPhase;
            }
            int pt=temparray[i-1]-phase*pointsPerPhase;
            int pt2=temparray[i]-phase2*pointsPerPhase;
            //printf("qhull_pt: %i phase: %i, point:%i, pointsPerPhase: %i\n",temparray[i-1],phase,pt,pointsPerPhase);
            //printf("qhull_pt2: %i phase2: %i, point2:%i, pointsPerPhase: %i\n",temparray[i],phase2,pt2,pointsPerPhase);
            
            
            //double yy_one = freeEnergy[temparray[i-1]];
            //double yy_two = freeEnergy[temparray[i]];
            int addone=1;
            int addtwo=1;
            if(addone==1)// || addone==2)
            {
                eq.xx[tangent]=	points[temparray[i-1]];
                eq.yy[tangent]= freeEnergy[temparray[i-1]];
                eq.phaseNumber[tangent]=temparray[i-1];

                tangent++;
            }
            if(addtwo==1)// ||addtwo==2)
            {
                eq.xx[tangent]= points[temparray[i]]; 
                eq.yy[tangent]= freeEnergy[temparray[i]];
                eq.phaseNumber[tangent]=temparray[i];
                
                //yy[tangent]= freeEnergy[temparray[i]];
                tangent++;
            }
            
            
            /*if(add==1 || add==0)
             {
             eq.xx[tangent]=	points[qhull_array[i-1]];
             eq.xx[tangent+1]= points[qhull_array[i]];
             eq.yy[tangent]= freeEnergy[qhull_array[i-1]];//printf("row: %i, point : %lf\n",i, points[qhull_array[i]] );
             eq.yy[tangent+1]= freeEnergy[qhull_array[i]];
             tangent+=2;
             }
             else
             {
             eq.nb_tangents--;
             }*/
            
		}
	}

    eq.nb_tangents=tangent;
    
    
    
    
    return eq;
}
qhull_tangent_t QHSolver2D(double T, double *points,int pointsLen,double *freeEnergy,int lenn,double *phaseLengths,int phaseLengthsLen,double largestInterval)
{
    if (pointsLen!=lenn)
        exit(100);  // data arrays are not the same lengths...
    /*setting up data for qhull function 
    / very important needs to be used to clear memory
    */
    int curlong, totlong;
    

    /*  variables represents total number of *phases*
     *  or anything else using the solver
     *  computed from phaseLengthsLen
     */
    int variables=phaseLengthsLen;
    /*
     *Number of dimensions for qhull
     *Set to two as this is for a binary solver --- does not mean two phases, just two dimensions
     *The points therefore just have an "x" coordinate and a value or "y" coordinate
     */
	int dim=2;
    
	int i;
	coordT *qpoints;
	facetT *facet;
	//Alloc qpoints array 
	if (!(qpoints = (double*)malloc ((dim*pointsLen)*sizeof(coordT)))) // allocates memory for an array that will copy a point(concentration) in an index and their corresponding free energy in the next index number.
	{ 
		printf ("qhullCalcHull: could not allocate memory for points\n");
		exit(1) ; 
	}
    //creating the qhull input array for two dimensions
	for (i=0; i<pointsLen; i++)
	{
		qpoints[dim*i]   = (coordT) points[i];
		qpoints[dim*i+1] = (coordT) freeEnergy[i];
	}
	
	//calling qhull
	qh_init_A (stdin, stdout, stderr, 0, NULL);
	qh_init_B (qpoints, pointsLen, dim, False);
	qh_qhull (); 
	//Free qhull input (we are trying to be memory efficient here)
	free(qpoints);
    
    int  qhull_array[qh num_facets * dim];
    
    int j=0,count=0;
	FORALLfacets
    {    
		vertexT *vertex, **vertexp;
		FOREACHvertex_(facet->vertices)
		{
			int thepoint = (vertex->point-qh first_point)/2.0;
			if(thepoint<pointsLen);
            qhull_array [count++] = thepoint;
		}
    }
	int k;
    
    double gap = largestInterval*1.1;
	for(k=0;k<count;k++)
	{
		for (j=k+1;j<count;j++)
		{
			if ( points[qhull_array[k]] > points[qhull_array[j]] )
			{
				int temp = qhull_array[k];
				qhull_array[k] = qhull_array[j];
				qhull_array[j] = temp;
			}
		}
	}
    qhull_tangent_t eq;
    eq.Temp=T;
    eq.nb_tangents=0;
    
    int temparray[count];
    int tempcounter=0;
    int phases;
    int addToArray=1;
    double yy_ones;
    int pts;
    
    
}