//
//    QHSolve.h
//    gibbs
//
//
//    Created by Thomas Cool on 1-Sept-2009.
//    Copyright 2009 Thomas Cool, R. Edwin Garcia
//    This file is part of gibbs.
//
//    gibbs is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/timeb.h>
#include <qhull/qhull_a.h>
#include <qhull/qset.h>
#include <qhull/qhull.h>

struct qhull_tangent_t
{
	int nb_tangents;
	double Temp;
	double *xx; //2 x and 2y per tangent
	double *yy;
    double *phaseNumber;
    //double *xhull;
    //double *yhull;
    //int count;
};
typedef struct qhull_tangent_t qhull_tangent_t;

struct array_struct_t
{
    int nbelements;
    double *array;
};
typedef struct array_struct_t array_struct_t;

struct mat_properties_t
{
	double omega;
	double delta_Ha;
	double delta_Hb;
	double temp_m_a;
	double temp_m_b;
};

typedef struct mat_properties_t mat_properties_t;
extern int calc(double *points,int len);
extern double getDouble (double *a,int b);

extern void	setDouble (double *a,int index, double value);
//extern double generatePointsArray( int nb_pts, int nb_elements,int totalmax, double min, double max);
extern void freeStruct(qhull_tangent_t structs);

extern void freeArray(double *arrays);

//extern array_struct_t createSArray(int nbelements);
extern int getPointsArrayN(double *points, int len, int nb_elements, int nb_pts, double min, double max);

extern void setDoubleInSA(array_struct_t s, int loc, double value);

extern double getDoubleInSA(array_struct_t s, int loc);
extern array_struct_t getPointsArraySA( int nb_pts, int nb_elements,int totalmax, double min, double max);

//array_struct_t createSArray(int nbelements);
//int getPointsArraySA(array_struct_t s, int nb_pts, int nb_elements,int totalmax, double min, double max);
array_struct_t getPointsArraySA(int nb_pts, int nb_elements,int totalmax, double min, double max);

void releaseDoubleArray(double *a);


extern void printToFile(double *points, double *freeEnergy, int numpoints);

extern mat_properties_t setMatStruct(double omega, double delta_Ha, double delta_Hb, double temp_m_a, double temp_m_b);

/***********************************
 / calculates the free energies for certain materials and puts them in the proper free energy array
 / points				array containing all the free energies for the materials		(see: getPointsArray)
 / freeEnergy			array to fill initialized with same amount of elements as points
 / nb_pts				is the total number of data points per material (ex: 200)
 / elt_number			number of the material (should not be bigger than nb_elements in getPointsArray
 / totalmax				is number of points for which points is malloc'ed.				(see: getPointsArray)
 **********************************/
extern int calculateFreeEnergyArrays(double *points, double *freeEnergy, int totalmax, int nb_pts, int elt_numbers, mat_properties_t material, double Temp );


/**************************
 / nb_pts				is the total number of data points per material (ex: 200)
 / nb_elements			is number of different materials (ex: 3)
 / totalmax				is number of points for which points is malloc'ed. It should be equal to nb_pts*nb_elements. 
 /						If it is bigger, it does not matter, if it is smaller, it will return a kOVERFLOWcatch error (1001)
 / points				is a double array alloc'ed with totalmax elements
 *************************/
extern int getPointsArray(double *points, int nb_pts, int nb_elements, int totalmax, double min, double max);


/**************************
 /function returning a qhull_tangent_t structure of all the tangent points Extremities
 ///return:
 /nb_tangents		integer representing the number of tangents found at certain temp
 /xx				array with all the x points for the tangents (2 points per tangent): if 3 tangents, 6 xx points
 /yy				array with all the y points for the tangents (2 points per tangent)
 /Temp				temperature at which the tangent was computed
 /
 //inputs
 /T					temperature
 /numpoints			number of elements * number of points per element
 /points			points x-coordinates: (see getPointsArray)
 /freeEnergy		freeEnergies corresponding to points Array (see calculateFreeEnergiesArray)
 **************************/
extern qhull_tangent_t TCQHSolver(double T, int numpoints,int variables, double *points,double *freeEnergy);
//extern qhull_tangent_t QHSolver(double T, int numpoints,int variables, double *points,double *freeEnergy);
extern qhull_tangent_t QHSolver(double T, int numpoints,int variables, double *points,int len,double *freeEnergy,int lenn);
extern qhull_tangent_t QHSolver2D(double T, double *points,int len,double *freeEnergy,int lenn,double *phaseLengths,int phaseLengthsLen,double largestInterval);


extern int cube( int n );
