from gibbs.solvers.qhsolver.qhsolvercore import QHSolverCore
from gibbs.solvers.qhsolver import QHSolve



class CommonTangentSolver(QHSolverCore):
    def __init__(self,x=[0,1,200],y=[700],x1=None,x2=None,x3=None,variables=None,DG=None,PB='CLI',**kwargs):
        QHSolverCore.__init__(self,x=x,y=y,variables=variables,DG=DG)
        if x1!=None:
            self.x_axe='x1'
        elif x2!=None:
            self.x_axe='x2'
        elif x3!=None:
            self.x_axe='x3'
        else:
            self.x_axe='x2'
            x2=0.0
        
        self.xl=[x1,x2,x3]
        self._solve()
    def _getFixVal(self):
        if self.x_axe=='x1':
            return self.xl[0]
        elif self.x_axe=='x2':
            return self.xl[1]
        else:
            return self.xl[2]
            
    def _calculate(self,points,yVal,varY,varX,stepsLoopX):
        #calcArray=QHSolve.new_doubleArrayFct(stepsLoopX*len(self.DG))
        from numpy import zeros
        calcArray=zeros(stepsLoopX*len(self.DG),'d')
        for i in range(len(self.DG)):

            self.DG[i].evalf(freeEnergyArray=calcArray,points=points,arraylength=stepsLoopX*len(self.DG),xsteps=stepsLoopX,phaseNumber=i,xl=self.xl,varX=varX,varY=varY,yVal=yVal)
        return calcArray
    def __call_(self,**kwargs):
        self._solve(**kwargs)
        
    def _solve(self,**kwargs):
        VarYVal,b,c=self._returnValY()

        minLoopX,maxLoopX,stepsLoopX = self._returnValX()
            
        phases=self._getNbPhases()
        varY=self._returnVariableY()
        varX=self._returnVariableX()
        
        stepsLoopX=int((1.0-self._getFixVal())*stepsLoopX)
        minLoopX=0.0
        maxLoopX=1.0-self._getFixVal()
        points = self._getPoints(phases,stepsLoopX,minLoopX,maxLoopX)         #get points array
        
    
        totalmax=stepsLoopX*phases                                                #total number of free energies for Qhull  
        freeEnergy = self._calculate(points,VarYVal,varY,varX,stepsLoopX)
        c=QHSolve.QHSolver(VarYVal,totalmax,phases,points,freeEnergy)                #run the convex hull solver itself

        for j in range(c.nb_tangents):                                              #add points to Lists
                self.P.append(QHSolve.getDouble(c.xx,j))
                self.L.append(c.Temp)
        self.P=points
        self.L=freeEnergy
        self.Px=[]
        self.Py=[]

        for j in range(c.nb_tangents):                                              #add points to Lists
            if self.x_axe=='x1':
                self.Px.append(self.xl[0])
                self.Py.append(QHSolve.getDouble(c.xx,j))
            elif self.x_axe=='x2':
                self.Px.append(QHSolve.getDouble(c.xx,j))
                self.Py.append(self.xl[1])
            else:
                self.Px.append(QHSolve.getDouble(c.xx,j))
                self.Py.append(1.0-self.xl[2]-QHSolve.getDouble(c.xx,j))
            
        self.steps=stepsLoopX
        QHSolve.freeStruct(c)
        
    def type(self):
        return '1Dimensional'
        
