#
#    commontangentsolver.py
#    gibbs
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#

from gibbs.solvers.qhsolver.qhsolvercore import QHSolverCore
from gibbs.solvers.qhsolver import QHSolve



class CommonTangentSolver(QHSolverCore):
    def __init__(self,x=[0,1,200],y=[650,1300,150],variables=None,DG=None,**kwargs):
        QHSolverCore.__init__(self,x=x,y=y,variables=variables,DG=DG)
        self._solve()
        
    def __call_(self,**kwargs):
        self._solve(**kwargs)
        
    def _solve(self,**kwargs):
        VarYVal,b,c=self._returnValY()

        minLoopX,maxLoopX,stepsLoopX = self._returnValX()
            
        phases=self._getNbPhases()
        varY=self._returnVariableY()
        varX=self._returnVariableX()
            
        points = self._getPoints(phases,stepsLoopX,minLoopX,maxLoopX)         #get points array
    
        totalmax=stepsLoopX*phases                                                #total number of free energies for Qhull  
        freeEnergy = self._calculate(points,VarYVal,varY,varX,stepsLoopX)
#        for i in range(totalmax):
#            print QHSolve.doubleArrayFct_getitem(points,i),QHSolve.doubleArrayFct_getitem(freeEnergy,i)
        c=QHSolve.QHSolver(VarYVal,totalmax,phases,points,freeEnergy)                #run the convex hull solver itself

        for j in range(c.nb_tangents):                                              #add points to Lists
                self.P.append(QHSolve.getDouble(c.xx,j))
                self.L.append(c.Temp)
        self.P=points
        self.L=freeEnergy
        self.T=VarYVal
        self.Px=[]
        self.Py=[]

        for j in range(c.nb_tangents):                                              #add points to Lists
            self.Px.append(QHSolve.getDouble(c.xx,j))
            self.Py.append(QHSolve.getDouble(c.yy,j))
            
        self.steps=stepsLoopX
        
    def type(self):
        return '1Dimensional'
    def write(self,filename='data.txt',add_constants={},append=False):
        FILE=open(filename,"w")
        FILE.write("########\n\n")
        FILE.write("columns: \n")
        FILE.write("0:C\n")
        FILE.write("1:Delta_G\n")
        FILE.write("2:Temp\n\n")
        FILE.write("########\n\n")
        for i in range(len(self.P)):
            FILE.write("%f  %f  %f\n"%(self.P[i],self.L[i],self.T))
        FILE.close()
        
