#
#    phasediagramsolver.py
#    gibbs
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#

from gibbs.solvers.qhsolver.qhsolvercore import QHSolverCore
from gibbs.solvers.qhsolver import QHSolve
import time

#import QHSolve


class PhaseDiagramSolver(QHSolverCore):
    def __init__(self,x=[0,1,200],y=[650,1300,150],variables=None,DG=None,ProgressBar='None',**kwargs):
        QHSolverCore.__init__(self,x=x,y=y,variables=variables,DG=DG,ProgressBar=ProgressBar)
        self._solve()


        
    def _solve(self, **kwargs):
        from gibbs.tools import ProgressBar
        from numpy import arange,argmin,argmax
        
        minLoopY,maxLoopY,stepsLoopY = self._returnValY()
        
        
        minLoopX,maxLoopX,stepsLoopX =self._returnValX()
            
 
        phases = self._getNbPhases()                                        #number of phases from FEParameterList
        varY=self._returnVariableY()
        varX=self._returnVariableX()
            
            
        points = self._getPoints(phases,stepsLoopX,minLoopX,maxLoopX)         #get points array
        totalmax=stepsLoopX*phases                                                #total number of free energies for Qhull        
        stepsize=float(maxLoopY-minLoopY)/stepsLoopY                            #stepsize for temperature increase
        self.stepsize=stepsize
        prog=ProgressBar(type=self.PB, min=minLoopY,max=maxLoopY)
            
        oldtan=0
        tempList=[]
        xtempList=[]
        for Temp in arange(minLoopY,maxLoopY,stepsize):                         #main loop
            prog(Temp)
            
            freeEnergy = self._calculate(points,float(Temp),varY,varX,stepsLoopX)
            t3=time.time()              
            c=QHSolve.QHSolver(Temp,totalmax,phases,points,freeEnergy)

            newPhaseL=[]
            newPhaseP=[]
            minV=0
            maxV=0
            for j in range(c.nb_tangents):
                newPhaseL.append(int(QHSolve.getDouble(c.phaseNumber,j))/int(stepsLoopX))
                newPhaseP.append(int(QHSolve.getDouble(c.phaseNumber,j)))
                
                
            ######## Triple Point Logic
            if (c.nb_tangents>oldtan or c.nb_tangents<oldtan) and c.nb_tangents%2==0 and c.nb_tangents!=0 and oldtan!=0:
                add=False
                if len(newPhaseL)>len(oldPhaseL):
                    newC=newPhaseL[:]
                    for p in oldPhaseL:
                        newPhaseL.remove(p)
                    newPhaseL.sort()
                    if len(newPhaseL)>1 and newPhaseL[0]==newPhaseL[len(newPhaseL)-1] and len(newPhaseL)%2==0:
                        nn=[]
                        for ii in oldPhaseP:
                            mm=[]
                            for jj in newPhaseP:
                                mm.append(abs(jj-ii))
                            nn.append(argmin(mm))
                        if len(nn)==2 and max(nn)-min(nn)==3:
                            add=True
                            mm=[]
                            for ii in oldPhaseP:
                                mm.append(points[ii])
                            maxV = max(mm)
                            minV = min(mm)
                        else:
                            add=False
                elif len(newPhaseL)<len(oldPhaseL):
                    newC=oldPhaseL[:]
                    for p in newPhaseL:
                        oldPhaseL.remove(p)
                    oldPhaseL.sort()
                    if len(oldPhaseL)>1 and oldPhaseL[0]==oldPhaseL[len(oldPhaseL)-1] and len(oldPhaseL)%2==0:
                        add=True
                        nn=[]
                        for ii in newPhaseP:
                            mm=[]
                            for jj in oldPhaseP:
                                mm.append(abs(jj-ii))
                            nn.append(argmin(mm))
                        if len(nn)==2 and (max(nn)-min(nn))==3:
                            add=True
                            mm=[]
                            for ii in newPhaseP:
                                mm.append(points[ii])
                            maxV = max(mm)
                            minV = min(mm)
                        else:
                            add=False
                if add==True:
                    tempList.append([c.Temp,c.Temp])
                    xtempList.append([minV,maxV])
                    
            ######### end triple point logic
            
            oldtan=c.nb_tangents
            oldtanL=[]
            oldPhaseL=[]
            oldPhaseP=[]
            for j in range(c.nb_tangents):                                             #add points to Lists
                    self.P.append(QHSolve.getDouble(c.xx,j))
                    self.L.append(c.Temp)
                    oldtanL.append(QHSolve.getDouble(c.xx,j))
                    oldPhaseL.append(int(QHSolve.getDouble(c.phaseNumber,j)/stepsLoopX))
                    oldPhaseP.append(int(QHSolve.getDouble(c.phaseNumber,j)))
            
            QHSolve.freeStruct(c)
            #QHSolve.delete_doubleArrayFct(freeEnergy)

        #QHSolve.delete_doubleArrayFct(points)
        self.tempList=tempList
        self.xtempList=xtempList

        prog.close()
        t2=time.time()

    def type(self):
        return '2Dimensional'

        

    
