#
#    qhsolvercore.py
#    gibbs
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#

from gibbs.solvers import SolverCore
from gibbs.variables.pyvariable.pyvariable import PyVariable
import QHSolve


class QHSolverCore(SolverCore):
    '''
    QHSolver Class
    solves the Convex Hull based on different options
    
    inputs:
    Nx      Number of points on x-axis
    phaseCalucations    Number of different Temperatures Computed
    maxT        Maximum Temperature computed
    minT        Minimum Temperature computed
    
    additional kwargs
    DG              List of different Sympy expressions representing 
                        free energies of different phases, should only have
                        Symbol x left
    
    Not Working Yet
    FEClass             free Energy computation class (usually SWIG)
    FEParameters        list of structures to pass to the FEClass
    '''
                        
    def __init__(self,x=[0,1,200],y=[650,1300,150],variables=None,DG=None,ProgressBar=None,**kwargs):
        SolverCore.__init__(self,x=x,y=y,variables=variables,DG=DG)
        self.Px=[]
        self.Py=[]
        
        if ProgressBar not in ['TK','CLI','None',None]:
            self.PB='None'
        else:
            self.PB=ProgressBar
            
            
        self.variables=variables                                                #variables for sympy
        
        
       	self.P=[]                                                               #horizontal data points for phase diagrams
        
        self.L=[]                                                               #vertical data points for phase diagrams

            
            
        #SaveFileOptions    
        if kwargs.has_key('file'):
            self.file=kwargs['file']
            self.save=True
        else:
            self.save=False

        
        self.autorun=1    
        self.g=None                                                      #start immediatly?
        
        #autorun: should it start computing at once (default yes) only for sympy computations
        if kwargs.has_key('autorun'):
            self.autorun=kwargs['autorun']


    def _calculate(self,points,yVal,varY,varX,stepsLoopX):
        #calcArray=QHSolve.new_doubleArrayFct(stepsLoopX*len(self.DG))
        from numpy import zeros
        calcArray=zeros(stepsLoopX*len(self.DG),'d')
        for i in range(len(self.DG)):

            self.DG[i].evalf(freeEnergyArray=calcArray,points=points,arraylength=stepsLoopX*len(self.DG),xsteps=stepsLoopX,phaseNumber=i,varX=varX,varY=varY,yVal=yVal)
        return calcArray
            
    def _getPoints(self,phases,Nx,min=0,max=1):
        from numpy import zeros
        points=zeros(Nx*phases,'d')
       	errorVal	= QHSolve.getPointsArrayN(points,phases,Nx,min,max)
       	return points
    def _solve(self,**kwargs):
        a=3.03
        
        
    def _getNbPhases(self):
        return len(self.DG)
        
    def Xdata(self):
        return self.P
        
    def Ydata(self):
        return self.L
        
    def XLineData(self):
        return self.Px
        
    def YLineData(self):
        return self.Py

