//    QHDataFileVariableMethods.c
//    gibbs
//
//
//    Created by Thomas Cool on 1-Sept-2009.
//    Copyright 2009 Thomas Cool, R. Edwin Garcia
//    This file is part of gibbs.
//
//    gibbs is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    gibbs is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#include "stdio.h"
#include "QHDataFileVariableMethods.h"
double linearInterpolate(double A[][3],double b[],double xvalue,double yvalue);
double detA(double A[][3]);
void interpolate1d(double *xdata, int xdataLen,    //passed as a numpy array from python - represents xdata or Composition 
                     double *ydata, int ydataLen,    //passed as a numpy array from python - represents ydata or Free Energy
                     double *fxdata, int fxdataLen,  //passed as a numpy array from python - represents the xdata loaded from file
                     double *fydata, int fydataLen,  //passed as a numpy array from python - represents the xdata loaded from file
                     int N,                         //Number of the phase - concretely, where to insert the interpolated data
                     int Nx,                        //Number of samplings across xdata - should correspond to xdataLen/(total Number of Phases)
                     int add                        //Determines if the energy should be added in addition to the existing data or not
                     )
{
    int i;
    int l;
    int fxp = 0;                                    //index of walker through fxdata
    int xstart=Nx*N;
    int meh;

    for (i=0; i<Nx; i++) {
        int index = i + N*Nx;
        double xvalue=xdata[index];
        double fxvalue=fxdata[fxp];
        while (fxvalue<=xvalue && fxp<=fxdataLen) {
            fxp++;
            fxvalue=fxdata[fxp];
        }
        
        //printf("%i\n",fxp);
        if(fxp>0 && fxdata[fxp-1]==xvalue)
            ydata[index]=fydata[fxp-1];
        else if(fxdata[fxp]==xvalue)
            ydata[index]=fydata[fxp];
        else if(fxp>0 && fxdata[fxp-1]<xvalue && fxdata[fxp]>xvalue)
        {
//            printf("in area where interpolation is needed\n");
//            printf("%lf,%lf,%lf\n",fxdata[fxp-1],fxdata[fxp],xvalue);
            float one = (fxdata[fxp]-xvalue)/(fxdata[fxp]-fxdata[fxp-1]);
            float two = (xvalue-fxdata[fxp-1])/(fxdata[fxp]-fxdata[fxp-1]);
            float sum = one+two;
            //printf("proportions: %lf %lf %lf",one,two,sum);
            ydata[index]=one*fydata[fxp-1]+two*fydata[fxp];
        }

        else if(fxp>0)
            l=0;
            //printf("%lf,%lf,%lf\n",fxdata[fxp-1],fxdata[fxp],xvalue);
        else
            l=1;
           // printf("i have no clue\n");
        if (fxp>=fxdataLen) {
            ydata[index]=fydata[fydataLen];
        }
        else if (fxp==0) {
            ydata[index]=fydata[0];
        }
            
        
            
    }
}
void interpolate2d(double *xdata, int xdataLen,    //passed as a numpy array from python - represents xdata or Composition 
                   double *ydata, int ydataLen,    //passed as a numpy array from python - represents ydata or Free Energy
                   double *fxdataU, int fxdataLenU,  //passed as a numpy array from python - represents the xdata loaded from file
                   double *fydataU, int fydataLenU,  //passed as a numpy array from python - represents the ydata loaded from file for the below
                   double *fxdataA, int fxdataLenA,
                   double *fydataA, int fydataLenA,
                   double below, double above, double val,
                   int N,                         //Number of the phase - concretely, where to insert the interpolated data
                   int Nx,                        //Number of samplings across xdata - should correspond to xdataLen/(total Number of Phases)
                   int add                        //Determines if the energy should be added in addition to the existing data or not
                   )
{
    if (add!=1) {
        add=0;
    }
    int i;
    int l;
    int fxpU = 0;                                    //index of walker through fxdataU
    int fxpA = 0;
    int xstart=Nx*N;
    
    for (i=0; i<Nx; i++) 
    {
        int index = i + N*Nx;
        double xvalue=xdata[index];
        double yvalue=ydata[index];
        //printf("xvalue: %lf\n",xvalue);
        //Finding the Index of the under array
        double fxvalue=fxdataU[fxpU];
        while (fxvalue<=xvalue && fxpU<(fxdataLenU-1)) {
            fxpU++;
            fxvalue=fxdataU[fxpU];
        }
        //printf("fxvalueU:%lf",fxvalue);
        //Finding the Index of the above array
        fxvalue=fxdataA[fxpA];
        while (fxvalue<=xvalue && fxpA<(fxdataLenA-1)) {
            fxpA++;
            fxvalue=fxdataA[fxpA];
        }
        //printf("fxvalueA:%lf",fxvalue);
        double value = 0.0;
        if (fxdataA[fxpA]<=xvalue && fxdataU[fxpU]<=xvalue) {
            //printf("here2\n");
            value= (above-val)/(above-below)*fxdataU[fxpU]+(val-below)/(above-below)*fxdataA[fxpA];
        }
        else if (fxdataA[fxpA]<=xvalue)
        {
            //Triangle interpolation (two points on Under)
        }
        else if (fxdataU[fxpU]<=xvalue)
        {
            //Triangle interpolation (two points on Above)
        }
        else if(fxpA==0 && fxpU==0)
        {
            value= (above-val)/(above-below)*fxdataU[fxpU]+(val-below)/(above-below)*fxdataA[fxpA];
        }
        else if(fxpA==0)
        {
            //Triangle interpolation (two points on Under)
        }
        else if(fxpU==0)
        {
            //Triangle interpolation (two points on Above)
        }
        else
        {
            //printf("in box\n");
            double yline=(above-below)/(fxdataA[fxpA-1]-fxdataU[fxpU])*(xvalue-fxdataA[fxpA-1])+above;
            //double A[3][3];
            double interpolatedValue;
            if(yline<=val)
            {
//                printf("top right triangle      %lf\n",yline);
                double A[3][3]={{fxdataA[fxpA-1],above,1},{fxdataA[fxpA],above,1},{fxdataU[fxpU],below,1}};
                double b[3]={fydataA[fxpA-1],fydataA[fxpA],fydataU[fxpU]};
                interpolatedValue=linearInterpolate(A, b, xvalue, val);
//                printf("xvalue: %lf %lf %lf %lf\n",xvalue,fxdataA[fxpA-1],fxdataA[fxpA],fxdataU[fxpU]);
//                printf("yvalue: %lf %lf %lf %lf\n",interpolatedValue,fydataA[fxpA-1],fydataA[fxpA],fydataU[fxpU]);
//                printf("val: %lf above: %lf %lf under: %lf\n",interpolatedValue,b[0],b[1],b[2]);
            }
            else 
            {
//                printf("bottom left triangle    %lf\n",yline);
                double A[3][3]={{fxdataA[fxpA-1],above,1},{fxdataU[fxpU-1],below,1},{fxdataU[fxpU],below,1}};
                double b[3]={fydataA[fxpA-1],fydataU[fxpU-1],fydataU[fxpU]};
                interpolatedValue=linearInterpolate(A, b, xvalue, val);
                
            }
            value=interpolatedValue;
            //double A[3][3]={1,2,1,4,4,1,7,9,1};
            

            //value=fydataU[fxpU-1]*(fxdataA[fxpU]-xvalue)*(above-val)/(fxp
        }
        ydata[index]=value;
//        printf("index: %i, indexU: %i, indexA: %i value: %lf\n",i,fxpU,fxpA,value);
    }
//    printf("lengths: %i, %i\n",fxdataLenU,fxdataLenA);
}

double linearInterpolate(double A[][3],double b[],double xvalue,double yvalue)
{
        int i,j;
////    printf("######\n");
//        for(i=0;i<3;i++)
//            for(j=0;j<3;j++)
//                printf("%lf ",A[i][j]);
//    printf("\n");
    double detV=detA(A);
//    printf("detV: %lf\n",detV);
    //invert matrix
    double At[3][3];
    for(i=0;i<3;i++)
        for(j=0;j<3;j++)
            At[j][i]=A[i][j];
//    for(i=0;i<3;i++)
//        for(j=0;j<3;j++)
//            printf(" ");
//    printf("%i %i",i,j);
    //printf("\n");
    double AdjA[3][3];
    AdjA[0][0]=At[1][1]-At[1][2];
    AdjA[0][1]=At[1][0]-At[1][2];

    AdjA[0][2]=At[1][0]-At[1][1];
    //printf("%lf %lf\n",At[1][0]-At[1][1],AdjA[0][2]);

    AdjA[1][0]=At[0][1]-At[0][2];
    AdjA[1][1]=At[0][0]-At[0][2];
    AdjA[1][2]=At[0][0]-At[0][1];
    AdjA[2][0]=At[0][1]*At[1][2]-At[0][2]*At[1][1];
    AdjA[2][1]=At[0][0]*At[1][2]-At[0][2]*At[1][0];
    AdjA[2][2]=At[0][0]*At[1][1]-At[0][1]*At[1][0];

    for(i=0;i<3;i++)
        for(j=0;j<3;j++)
            if ((i+j)%2!=0)
                AdjA[i][j]=AdjA[i][j]*-1.0f;
//    for(i=0;i<3;i++)
//        for(j=0;j<3;j++)
//            printf("%lf ",AdjA[i][j]);
//    printf("\n");
                
                
    double invA[3][3];
    
    for(i=0;i<3;i++)
    {
        for(j=0;j<3;j++)
        {
            invA[i][j]=AdjA[i][j]/detV;
        }
    }
//    for(i=0;i<3;i++)
//        for(j=0;j<3;j++)
//            printf("%lf ",invA[i][j]);
//    printf("\n");
    double Acoeff=0.0;
    double Bcoeff=0.0;
    double Ccoeff=0.0;
    for(i=0;i<3;i++)
    {
        Acoeff+=invA[0][i]*b[i];
    }
    for(i=0;i<3;i++)
    {
        Bcoeff+=invA[1][i]*b[i];
    }
    for(i=0;i<3;i++)
    {
        Ccoeff+=invA[2][i]*b[i];
    }
    double zvalue=Acoeff*xvalue+Bcoeff*yvalue+Ccoeff;
    //printf("z1 %lf,z2 %lf ,z3 %lf ,z: %lf\n",b[0],b[1],b[2],zvalue);
    return zvalue;
    
}

double detA(double A[][3])
{
//    int i=0;
//    int j=0;
//    for(i=0;i<3;i++)
//        for(j=0;j<3;j++)
//            printf("%lf ",A[i][j]);
//    printf("\n");
    double det= A[1][1]*A[0][0]+A[0][1]*A[2][0]+A[2][1]*A[1][0]-
                A[2][1]*A[0][0]-A[0][1]*A[1][0]-A[1][1]*A[2][0];
//    for(i=0;i<3;i++)
//        for(j=0;j<3;j++)
//            printf("%lf ",A[i][j]);
//    printf("\ndet: %lf",det);
//    printf("\n");
    return det;
}