#    filevariable.py
#    gibbs
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#

from gibbs.tools.utils import *
from gibbs.variables.variable import Variable
import QHDataFileVariableMethods as QHData
from numpy import where, array, arange
import copy
#from gibbs.variables.filevariable import FileVariable
#a=FileVariable('hello.txt')
#a.author()
class DataFileVariable(Variable):
    def __init__(self,filename=None,**kwargs):
        Variable.__init__(self,**kwargs)
#        if filename==None:
#            raise CustomException('No File Chosen')
        self.filename=filename
        self.constants={}
        self.Columns=[]
        self.ColumnTypes=[]
        self.data={}
        if filename!=None:
            self._parse(filename)
        #print self.constants
    def __str__(self):
        return "FileVariable(%s)"%self.filename
        
    def __copy__(self):
        import copy
        a=DataFileVariable()
        a.filename=self.filename
        a.constants=copy.deepcopy(self.constants)
        a.Columns=copy.deepcopy(self.Columns)
        a.ColumnTypes=copy.deepcopy(self.ColumnTypes)
        a.data=copy.deepcopy(self.data)
        a.info=copy.deepcopy(self.info)
        return a
        
    def copy(self):
        return self.__copy__()
    
    def _parse(self,filename):
        file = open(filename,'r')
        lines=file.readlines()
        file.close()
        lineCount=len(lines)
        i=0
        line=lines[i]
        self.title=line[:-1]
        i=i+1
        line=lines[i]
        #print line
#        if line.find("######")==-1:
#            raise CustomException('Second Line should be a seperator')
        i=2
        type=None
        while i<lineCount:
            line=lines[i]
            if line[-1]=="\n":
                line=line[:-1]
            if line[0]=='#':
                pass
            elif line.find("%constants")!=-1:
                type='Constants'
            elif line.find("%columns")!=-1:
                type='Columns'
            elif line.find("%data")!=-1:
                type='Data'
            elif line.find("######")!=-1:
                type=None
            else:
                #print type
                self._add(line,type)
            i=i+1
        for i in self.data.keys():
            self.data[i]=array(self.data[i])
    
    def _add(self,line,type):

        if type==None:
            return
        elif type=='Constants':
            i1=line.find('(')
            i2=line.find(')')
            i3=line.find(':')
            self.constants[line[:i1]]=self._cast(line[i3+1:],line[i1+1:i2])
        elif type=='Columns':
            i1=line.find('(')
            i2=line.find(')')
            self.Columns.append(line[:i1])
            self.ColumnTypes.append(line[i1+1:i2])
        elif type=='Data':
            #data=line.split('\t')
            #print line
            data=line.strip().split()
            #print data
            if len(self.data.keys())==0:
                self._setupColumns()
            if len(data)!=len(self.ColumnTypes):
                data=line.strip().split()
                #print data
                if len(data)!=len(self.ColumnTypes):
                    raise CustomException('data is not of correct length')
            
            
            for i in range(len(self.Columns)):
                self.data[self.Columns[i]].append(self._cast(data[i],self.ColumnTypes[i]))
            
    def __call__(self,**kwargs):
        from gibbs.tools.numpyextensions import intersect1dmult
        import copy
        datakeys=[]
        
        
        for i in kwargs.keys():
            if i in self.Columns:
                datakeys.append(i)
        if len(datakeys)==0:
            return self.data
            
        newdata=copy.deepcopy(self.data)
        indices=[]
        for key in datakeys:
            type=self._typeForKey(key)
            indices.append(where(array(self.data[key])==kwargs[key])[0])
        #print indices

        intersect=intersect1dmult(*indices)
        keys=self.data.keys()
        a={}
        constants=copy.deepcopy(self.constants)
        for key in keys:
            a[key]=self.data[key][intersect]
        for key in datakeys:
            del a[key]
            constants[key]=kwargs[key]
            
        returnObj=DataFileVariable()
        returnObj.filename=self.filename
        returnObj.data=a
        returnObj.constants=constants
        returnObj.Columns=copy.copy(self.Columns)
        returnObj.ColumnTypes=copy.copy(self.ColumnTypes)
        return returnObj
        
                
            
        
    def _typeForKey(self,key):
        if self.constants.has_key(key):
            return type(self.constants[key])
        elif self.data.has_key(key):
            i= self.Columns.index(key)
            return self.ColumnTypes[i]
            
    def levels(self,key=None):
        if key==None:
            return None
        if self.constants.has_key(key):
            return self.constants[key]
        elif self.data.has_key(key):
            a=list(set(self.data[key]))
            return a
        
                
    def _setupColumns(self):
        for i in self.Columns:
            self.data[i]=[]

    def __getattr__(self,name):
        if self.constants.has_key(name):
            return self.constants[name]
        elif self.data.has_key(name):
            return self.data[name]
        else:
            return None
            
    def evalf(self,points,yVal,varY,varX,xsteps,phaseNumber,freeEnergyArray,add=0,extraVars=None,interpolationVariable=None,interpolationValue=None,**kwargs):
        sdata=copy.deepcopy(self.data)
#        print len(sdata)
#        print sdata
        if len(sdata)>3:
            raise CustomException("Currently, the you can only interpolate over one variable: There can therefore only be a maximum of two variables left in the data")
        if len(sdata)<2:
            raise CustomException("There needs to be at least two variables")
        if len(sdata)==2:
            print "data cannot be dependent on both x and y as there are only two variables available"
        
#        print varY,yVal
#        print sdata.keys()
#        print len(sdata['C'])
        needs2DInterpolation=True
        if len(sdata)==2:
            needs2DInterpolation=False
        if needs2DInterpolation:
            indices=where(array(sdata[varY])==yVal)[0]
#            print 'len indices:',len(indices)
#            print indices
            if len(indices)==0:
                #print "needs 2D interpolation"
                needs2DInterpolation=True

            else:
                #print "only needs 1D interpolation"
                needs2DInterpolation=False
                keys=sdata.keys()
                for key in keys:
                    sdata[key]=sdata[key][indices]
                del sdata[varY]
#        print len(sdata)
#        print sdata.keys()
#        print len(sdata['C'])
#        print type(sdata['C'])
#        print varX
        a=sdata.keys()
        varV=''
        for i in a:
            if i not in [varX,varY]:
                varV=i
            
        
            
        if add!=1:
            add=0  
            

        if len(sdata)==2:
            #print 'passing'
            QHData.interpolate1d(points,freeEnergyArray,sdata[varX],sdata[varV],phaseNumber,xsteps,add)
            #this is where you do a 1D interpolation to get to the proper points
            pass
        elif len(sdata)==3:
            above= where(array(sdata[varY])>yVal)
            aboveVal=min(sdata[varY][above])
            
            under= where(array(sdata[varY])<yVal)
            underVal=max(sdata[varY][under])
            indicesUnder=where(array(sdata[varY])==underVal)[0]
            indicesAbove=where(array(sdata[varY])==aboveVal)[0]
#            print aboveVal,underVal
            
            QHData.interpolate2d(points,freeEnergyArray,sdata[varX][indicesUnder],\
                                sdata[varV][indicesUnder],sdata[varX][indicesAbove],\
                                sdata[varV][indicesAbove],float(underVal),float(aboveVal),float(yVal),phaseNumber,xsteps,add)
            #raise CustomException('done for first part')
            #this is where a 2D interpolation is done
            pass
        
#
#        if self._variableExists(interpolationVariable):
#            type=self._typeForKey(interpolationVariable)
#            if type==None:
#                indices=range(len(datacopy[datacopy.keys[0]]))
#            elif type=='str':
#                indices=where(datacopy.interpolationVariable==interpolationValue)[0]
        #raise CustomException("End For Now")     
                
    def _variableExists(self,key):
        if self.data.has_key(name):
            return True
        else:
            return False
                
                
    def _cast(self,value,type):
        #print type
        if type not in ['int','float','str','string']:
            type='str'
        if type in ['string', 'str']:
            type='str'
        if type=='str':
            return str(value)
        elif type=='int':
            return int(value)
        elif type=='float':
            return float(value)
                
                
        
        