#    pyvariable.py
#    gibbs
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
from gibbs.solvers.qhsolver import QHSolve
from sympy import Symbol
from sympy.core.basic import Basic
from sympy.core.add   import Add
from gibbs.variables.variable import Variable

class PyVariable(Variable):
    def __new__(cls, *args, **kwargs):
        from sympy.core import Basic
        if isinstance(args[0],Basic):
            obj=Variable.__new__(cls)
            return obj
        else:
            return Symbol(args[0])
            
    '''
    creates PyVariable instance
    only requires 1 item: an object
    '''
    def __init__(self,object,**kwargs):
        Variable.__init__(self,list=None)
        if isinstance(object,Basic):
            self.object=object
    
    '''
    Permanently Replaces all keys in kwargs with the corresponding values:
    for examples if kwargs has :
        x=0.3 and the object is x*3+2b, it will replace it by 0.9+3*b
    '''
    def __call__(self,**kwargs):
        a=kwargs.keys()
        b={}
        for i in a:
            if (not isinstance(kwargs[i],(Basic,Symbol,float,int,type([])))):
                b[i]=kwargs[i]
                del kwargs[i]
        self.object=self._subs(**kwargs)
        if (len(b)!=0):
            try:
                import fipy.variables as fipyvariables
                continueT=True
                for i in b.keys():
                    if(isinstance(b[i],fipyvariables.Variable)):
                        vars()[i]=b[i]
                    else:
                        continueT=False
                if continueT:
                    #print str(self.object)                  #can only be a sympy object very limited
                    returnVal=eval(str(self.object))        #here is eval
                    #print type(returnVal)
                    return returnVal
                else:
                    print "something went wrong in conversion to Fipy, someone needs to clarify this message" 
                
            except:
                print "not testing for fipy, cannot find fipy"

        else:
            return self.object
            #self.object=PyVariableCore(a)
            
    '''
    Evaluates the points into arrays ( the replacements are not permanent, a copy is created)
    yVal        = is a single float or int
    points      = Numpy array containing all Phases
    varY        = either PyVariable, Sympy Symbol or str corresponding to Y-axis
    varX        = same as varY but for X-axis
    xsteps      = number of sampling of points on x-axis
    phaseNumber = which part of the freeEnergyArray to fill:
                  if there are three phases, 200 xsteps and phaseNumber=2,
                  it will fill the freeEnergyArray from point 200 to 399
    add         = if add=1, it will add the calculated to the one inplace
                  if add=0, it will replace it
    extraVars   = dictionary of additional keys to replace
    '''
    def _subs(self,object=None,**kwargs):
        if object==None:
            a=self.object
        else:
            a=object
        if len(kwargs)!=0:
            for i in kwargs.keys():
                a=a.subs(i,kwargs[i])
        return a
            
            
    def evalf(self,points,yVal,varY,varX,xsteps,phaseNumber,freeEnergyArray,add=0,extraVars=None,**kwargs):
        if add!=1:
            add=0
        kwargs[str(varY)]=yVal
        tempobject=self._subs(**kwargs)
        #tempobject=PyVariableCore(tempobject(**kwargs))
        a={}
        if type(extraVars)==type([]) and len(extraVars)!=0:
            for i in extraVars:
                if type(i)==type([]):
                    a[str(i[0])]=i[1]
            
        tempobject=self._subs(object=tempobject,**a)

        args2={}
        for j in range(xsteps): 
            args2[str(varX)]=float(points[phaseNumber*xsteps+j])
            if add==1:
                freeEnergyArray[phaseNumber*xsteps+j]=freeEnergyArray[int(phaseNumber*xsteps+j)]+float((self._subs(object=tempobject,**args2)).evalf())
            else:
                freeEnergyArray[int(phaseNumber*xsteps+j)]=float((self._subs(object=tempobject,**args2)).evalf())

    '''
    string representation of the variable
    '''
    def __str__(self):
        return str(self.object)
        
    def __getattr__(self,name):
        return getattr(self.object,name)
        
    def diff2(self,name):
        return self.object.diff(name.object)
        
    def __copy__(self):
        return PyVariable(self.object,**self.info)
        

        
        
        
