//    QHTernaryRegularFreeEnergy.c
//    gibbs
//
//
//    Created by Thomas Cool on 1-Sept-2009.
//    Copyright 2009 Thomas Cool, R. Edwin Garcia
//    This file is part of gibbs.
//
//    gibbs is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.


#include "QHTernaryRegularFreeEnergyC.h"

double xlogx (double a)
{
	
	if  (a==0.0)
	{//printf("!!");
        return 0;}
	else if (a == 1.0)
	{	//printf("@@");
        return 0;}
	else if( a>0 && a<1)
		//printf(" value %f %f\n",a, log(a));
		return a*log(a);
	return 0.0;
}
double getDouble (double *a,int b)
{
	return a[b];
}
void setDouble(double *a, int index, double value)
{
	a[index]=value;
	//return 0;
	
}
double calcValue(double x_one, double x_two, double x_three, double Temp, double R, tern_reg_structP phase)
{
    double value;
    value = 
    (x_one)*(phase.delta_Ha - Temp*phase.delta_Ha/phase.Tma)+
    x_two*(phase.delta_Hb - Temp*phase.delta_Hb/phase.Tmb)+
    (x_three)*(phase.delta_Hc - Temp*phase.delta_Hc/phase.Tmc)+
    R*Temp*xlogx(x_one)+
    R*Temp*xlogx(x_three)+
    R*Temp*xlogx(x_two) +
    phase.omega_ot*x_one*x_two+
    phase.omega_tth*x_two*x_three+
    phase.omega_oth*x_one*x_three+
    phase.omega_otth*x_one*x_three*x_two;
    return value;
}
int calculateFreeEnergy(double *points,int len, double *freeEnergy, int lenn, double x_two, int nb_pts, int elt_number, tern_reg_structP phase, double Temp, int add )
{
    return calculateFreeEnergy_two(points,len,freeEnergy,lenn,x_two,nb_pts,elt_number,phase,Temp,add );
}

int calculateFreeEnergy_two(double *points,int len, double *freeEnergy, int lenn, double x_two, int nb_pts, int elt_number, tern_reg_structP phase, double Temp, int add )
{
    // Sanity check
    //int totalmax=len;
	double R=8.314;
	int i;
    double x_one,x_three;
	for(i=0;i<nb_pts;i++)
	{

		int index = i + elt_number*nb_pts;
        x_one=points[index];
        x_three=1.0-x_two-x_one;
        //Sanity Check:
        double value;
        value = calcValue(x_one, x_two, x_three, Temp, R, phase);

        if (add==1) { freeEnergy[index] += value;}
        else        { freeEnergy[index] =  value;}

	}
	return 0;
}
int calculateFreeEnergy_one(double *points,int len, double *freeEnergy, int lenn, double x_one, int nb_pts, int elt_number, tern_reg_structP phase, double Temp, int add )
{
    // Sanity check
    //int totalmax=len;
	double R=8.314;
    double x_two,x_three;
	int i;
	for(i=0;i<nb_pts;i++)
	{
        int index = i + elt_number*nb_pts;
        x_two=points[index];
        x_three=1.0-x_one-x_two;
        //Sanity Check:

        double value;
        value = calcValue(x_one, x_two, x_three, Temp, R, phase);

        if (add==1) { freeEnergy[index] += value;}
        else        { freeEnergy[index] =  value;}
        
	}
	return 0;
}

int calculateFreeEnergy_three(double *points,int len, double *freeEnergy, int lenn, double x_three, int nb_pts, int elt_number, tern_reg_structP phase, double Temp, int add )
{
    // Sanity check
    //int totalmax=len;
	double R=8.314;
    double x_two,x_one;
	int i;
	for(i=0;i<nb_pts;i++)
	{
        int index = i + elt_number*nb_pts;

        x_one=points[index];
        x_two=1.0-x_one-x_three;
        
        double value;
        value = calcValue(x_one, x_two, x_three, Temp, R, phase);
        if (add==1) { freeEnergy[index] += value;}
        else        { freeEnergy[index] =  value;}
        
	}
	return 0;
}

tern_reg_structP setPhase(double Tma, double Tmb, double Tmc,
                          double delta_Ha, double delta_Hb, double delta_Hc,
                          double omega_ot,double omega_tth,double omega_oth,
                          double omega_otth)
{
	tern_reg_structP b;
    b.Tma=Tma;
    b.Tmb=Tmb;
    b.Tmc=Tmc;
    b.delta_Ha=delta_Ha;
    b.delta_Hb=delta_Hb;
    b.delta_Hc=delta_Hc;
	b.omega_ot=omega_ot;        //Omega_{12}
    b.omega_tth=omega_tth;       //Omega_{23}
    b.omega_oth=omega_oth;       //Omega_{13}
    b.omega_otth=omega_otth;      //Omega_{123}
	return b;
}
