#    ternaryregularfreeenergy.py
#    gibbs
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#

from gibbs.variables.variablec import VariableC
import QHTernaryRegularFreeEnergyC as QHTRFEC

class TernaryRegularFreeEnergyVariable(VariableC):
    def __init__(self,delta_Ha,delta_Hb,delta_Hc,Tma,Tmb,Tmc,Omega12,Omega23,Omega13,Omega123,**kwargs):
        VariableC.__init__(self,**kwargs)
        self.delta_Ha=delta_Ha
        self.delta_Hb=delta_Hb
        self.delta_Hc=delta_Hc
        self.Tma=Tma
        self.Tmb=Tmb
        self.Tmc=Tmc
        self.Omega12=Omega12
        self.Omega23=Omega23
        self.Omega13=Omega13
        self.Omega123=Omega123
        
    '''
    Evaluates the points into arrays ( the replacements are not permanent, a copy is created)
    yVal        = is a single float or int
    points      = Numpy array containing all Phases
    xsteps      = number of sampling of points on x-axis
    phaseNumber = which part of the freeEnergyArray to fill:
                  if there are three phases, 200 xsteps and phaseNumber=2,
                  it will fill the freeEnergyArray from point 200 to 399
    add         = if add=1, it will add the calculated to the one inplace
                  if add=0, it will replace it
    extraVars   = dictionary of additional keys to replace
    '''   
    def evalf(self,freeEnergyArray,points,arraylength,xsteps,phaseNumber,yVal,add=0,xl=None,**kwargs):
        from numpy import zeros
        self.struct=QHTRFEC.setPhase(self.Tma,self.Tmb,self.Tmc,
                                    self.delta_Ha,self.delta_Hb,self.delta_Hc,
                                    self.Omega12,self.Omega23,self.Omega13,
                                    self.Omega123)
        if add!=0:
            add=1
            
        if xl==None:
            QHTRFEC.calculateFreeEnergy_two(points,freeEnergyArray,0.0,xsteps, phaseNumber,self.struct,float(yVal),add)
        elif len(xl)==3:
            if xl[0]==None and xl[1]==None and xl[2]==None:
                QHTRFEC.calculateFreeEnergy_two(points,freeEnergyArray,0.0,xsteps, phaseNumber,self.struct,float(yVal),add)
            elif xl[0]!=None:
                QHTRFEC.calculateFreeEnergy_one(points,freeEnergyArray,float(xl[0]),xsteps, phaseNumber,self.struct,float(yVal),add)
            elif xl[1]!=None:
                QHTRFEC.calculateFreeEnergy_two(points,freeEnergyArray,float(xl[1]),xsteps, phaseNumber,self.struct,float(yVal),add)
            elif xl[2]!=None:
                QHTRFEC.calculateFreeEnergy_three(points,freeEnergyArray,float(xl[2]),xsteps, phaseNumber,self.struct,float(yVal),add)

        
        #QHTRFEC.calculateFreeEnergy_three(points,freeEnergyArray,x_two,xsteps, phaseNumber,self.struct,float(yVal),add)
    
    '''
    string representation
    '''
    def __str__(self):
        return 'RegularTernaryFreeEnergyVariable(O=%s,DHa=%s,DHb=%s,Tma=%s,Tmb=%s)'%(self.omega,self.delta_Ha,self.delta_Hb,self.Tma,self.Tmb)