#    thermocalcdbvariable.py
#    gibbs
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gibbs is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gibbs.  If not, see <http://www.gnu.org/licenses/>.
#


#  TODO: parse functions
#        parse names
#        combine

from gibbs.tools.utils import *
from gibbs.variables.variable import Variable
class TCDBVariable(Variable):
    def __init__(self,database=None,R=8.314,**kwargs):
        Variable.__init__(self,**kwargs)
#        if filename==None:
#            raise CustomException('No File Chosen')
        self.database=database
        self.elements={}
        self.phases={}
        self.functions={}
        self.parameters={}
        self.phase=None
        self.phaseElts=[]
        if database!=None:
            self._parse(database)
        self.tRange=[None,None]
        self.eqStr=None
        self.prettyE=None
        self.R=R
        #print self.constants
    def __str__(self):
        if isinstance(self.phase,str) and len(self.phaseElts)>0:
            return "TCDBVariable(%s:%s)"%(self.phase,self.phaseElts)
        elif isinstance(self.phase,str):
            return "TCDBVariable(%s)"%self.phase
        if isinstance(self.database,str):
            return "TCDBVariable(%s)"%self.database
        else:
            return "TCDBVariable(Null)"
    
    def _parse(self,database):
        import os
        #print os.listdir(os.curdir)
        #print os.getcwd()

        file = open(database,'r')
        lines=file.readlines()
        file.close()
        lineCount=len(lines)
        counter=0
        type=None
        while (counter < lineCount):
            line=lines[counter]
            lineA=line.split()
            if len(lineA)>0 and line[0]!='$':
#                if '!' in lineA[-1]:
#                    lineEnd=False
#                    while lineEnd==False:
#                        counter=counter+1
#                        line=line+lines[counter]
#                        lineA=line.split()
#                        #print lineA,lineA[-1]
#                        if '!' in lineA[-1]:
#                            lineEnd==True
#                            print counter
#                    #print line
#                    
                if lineA[0] in "ELEMENT":
                    #print line[:-1]
                    #print lineA
                    element={'name':lineA[1],'ref. state':lineA[2],'mass':lineA[3],'H298':lineA[4],'S298':lineA[5]}
                    self.elements[lineA[1]]=element
                elif lineA[0] in "PHASE":
                    b=lineA[1].find(':')
                    if b==-1:
                        name=lineA[1]
                        bit=None
                        namek=lineA[1]
                    else:
                        name=lineA[1]
                        bit=lineA[1][b+1:]
                        namek=lineA[1][:b]
                    nOS=int(lineA[3])
                    sublattices=[]
                    for z in range(nOS):
                        sublattices.append(lineA[4+z])
                    phase={'name':name,'bit':bit,'data-type':lineA[2],'nbSublattices':nOS,'sublattices':sublattices,'constituents':[],'parameters':{}}
                    self.phases[namek]=phase
                elif lineA[0] in "CONSTITUENT":
                    name=lineA[1]
                    if name.find(":")!=-1:
                        name=name[:name.find(":")]
                    if name not in self.phases.keys():
                        print 'database seems to be wrongly formed'
                       # print name
                    else:
                        #line.find(':')
                        for i in range(2,len(lineA)):
                            csplit=lineA[i].split(':')
                            for j in csplit:
                                if len(j)!=0 and j!='!':
                                    if line.find("%"):
                                        t=self._parseConst(j)
                                        self.phases[name]['constituents'].append(t)
                                    else:
                                        self.phases[name]['constituents'].append(j)#=self.phases[name]['constituents']+csplit
                elif lineA[0] in "FUNCTION":
                    counter,line,lineA=self._combineLine(counter,lines,line,lineA)
#                    if "!" not in lineA[-1]:
#                        #print lineA
#                        lineEnd=False
#                        while lineEnd==False:
#                            counter=counter+1
#                            line=line+lines[counter]
#                            lineA=line.split()
#                            if "!" in lineA[-1]:
#                                lineEnd=True
#                    else:
#                        pass
                        #print lineA
                    name=lineA[1]
                    self.functions[name]=self._parseFunction(lineA)
                    
                elif lineA[0] in "PARAMETER":
                    counter,line,lineA=self._combineLine(counter,lines,line,lineA)
                    name=lineA[1]
                    self.parameters[name]=self._parseFunction(lineA)
                    nameD=self._parseParameterName(name)
                    self.parameters[name]['nameInfo']=nameD
                    #print nameD
                    self.phases[nameD['phase']]['parameters'][name]=self.parameters[name]
                    #print self.phases[nameD['phase']]

                    #print "parameter"
                    
            counter=counter+1
            
    def _combineLine(self,counter,lines,line,lineA):
        if "!" not in lineA[-1]:
            lineEnd=False
            while lineEnd==False:
                counter+=1
                line+=lines[counter]
                lineA=line.split()
                if "!" in lineA[-1]:
                    lineEnd=True
        return counter,line,lineA
                
            
    def _parseFunction(self,lineA):
        function={'name':lineA[1],'minTemp':lineA[2],'expressions':[],'maxTemps':[]}
        index=3
        done = False
        while done == False:
            doneLine=False
            eq=lineA[index]
            while doneLine==False:
                t=eq.find(';')
                if t==-1:
                    index+=1
                    eq=eq+lineA[index]
                elif t!=(len(eq)-1):
                    doneLine=True
                    #print eq.split(';')[-1]
                    function['expressions'].append(eq.split(';')[0])
                    #function['maxTemps'].append(None)
                    function['maxTemps'].append(30000)
                else:
                    doneLine=True
                    eq=eq[:-1]
                    function['expressions'].append(eq)
                    index+=1
                    function['maxTemps'].append(lineA[index])
            if "!" in lineA[index]:
                done=True
            else:
                index+=1
                if 'Y' in lineA[index]:
                    #print lineA[index]
                    index+=1
                if lineA[index] in ["N","REF:"]:
                    done=True
#            
#        
#            
#                    
#        else:
#            function['maxTemps'].append(None)
#        function['functions'].append(t[0])
        #print function,'\n\n'
            
            
            
        return function
    def _parseConst(self,data):
        split=data.split(",")
        t=[]
        for j in split:
            if j[-1]=="%":
                t.append(j[:-1])
            else:
                t.append(j)
        #print split
        return t
    def _parseParameter(self,line):
        parameter={'name':lineA[1],'minTemp':lineA[2],'expressions':[],'maxTemps':[]}
        index=3
        done = False
        
    def _parseParameterName(self,name):
        nameD={}

        nameD['type']=name[0]
        index1=name.find('(')
        index2=name.find(')')
        #print index2, index1
        newname=name[index1+1:index2]
        newname.find(';')
        nameD['power']=newname[newname.find(';')+1]
        newname=newname[:newname.find(';')]
        if newname.find(':')!=-1:
            newname=newname.split(':')
            subL1=newname[0].split(',')
            subL1.reverse()
            nameD['phase']=subL1.pop()
            subL1.reverse()
            subL2=newname[1].split(',')
            nameD['components']=[subL1,subL2]
        else:
            Lat=newname.split(',')
            Lat.reverse()
            nameD['phase']=Lat.pop()
            Lat.reverse()
            nameD['components']=[Lat]
        #print nameD
        return nameD
            
    
    def Phases(self):
        return self.phases.keys()
    
    def Elements(self):
        return self.elements.keys()
        
    def Components(self):
        if self.phase==None:
            return self.elements.keys()
        else:
            return self.phases[self.phase]['constituents']
        
    def Functions(self):
        return self.functions.keys()
        
    
    def Parameters(self):
        return self.parameters.keys()
        
    def setPhase(self,phase=None):
        a=self.copy()
        if phase==None:
            a.phase=None
            a.phaseElts=None
        elif phase in a.phases.keys():
            a.phase=phase
            if len(a.phases[phase]['constituents'][0])==2:
                a.phaseElts=a.phases[phase]['constituents'][0]
            else:
                a.phasesElts=None
        else:
            a.phase=None
            a.phaseElts=None
        return a
            
    def __copy__(self):
        return self.extractCopy()
    def copy(self):
        return self.extractCopy()
            
    def extractCopy(self):
        import copy
        copyObj=TCDBVariable()
        copyObj.database=self.database
        copyObj.elements=copy.deepcopy(self.elements)
        copyObj.phases=copy.deepcopy(self.phases)
        copyObj.functions=copy.deepcopy(self.functions)
        copyObj.parameters=copy.deepcopy(self.parameters)
        copyObj.phase=copy.copy(self.phase)
        copyObj.phaseElts=copy.copy(self.phaseElts)
        copyObj.R=self.R
        copyObj.info=copy.deepcopy(self.info)
        return copyObj

        
            
            
    def setComponents(self,components=None):
        if self.phase==None:
            self.phaseElts=None
        elif not isinstance(components,list):
            self.phaseElts=None
        elif len(components)!=2:
            CustomException("only 2 components for now")
        elif components[0] in self.phases[self.phase]['constituents'][0] and components[1] in self.phases[self.phase]['constituents'][0]:
            self.phaseElts=components
        else:
            print "elements not in currently selected phase"
            if len(self.phases[self.phase]['constituents'][0])==2:
                self.phaseElts=self.phases[phase]['constituents'][0]
            else:
                self.phasesElts=None
            
            
    def _replaceFunctions(self,tRange,equationStr,yVal):
        done=False
        while done==False:
            done=True
            for p in self.functions.keys():
                if equationStr.find(p)!=-1:
#                    print self.functions[p]
                    f=self.functions[p]
                    if yVal<float(f['minTemp']) or yVal>float(f['maxTemps'][-1]):
                        CustomException("outside of Temp range for functions: %s"%p)
                    found=False
                    index=0
                    while found==False:
                        if float(f['maxTemps'][index])>yVal:
                            found=True
                            if tRange[1]==None or tRange[1]>float(f['maxTemps'][index]):
                                tRange[1]=float(f['maxTemps'][index])
                            if index>0:
                                if float(f['maxTemps'][index-1])>tRange[0]:
                                    tRange[0]=float(f['maxTemps'][index-1])
                            elif index==0:
                                if float(f['minTemp'])>tRange[0]:
                                    tRange[0]=float(f['minTemp'])
                        else:
                            index+=1
                    #print "replacing %s by %s at %.2f"%(p,f['expressions'][index],yVal)
                    equationStr=equationStr.replace(p,f['expressions'][index])
                    done=False
#                    print p
#                    print equationStr
        return tRange,equationStr
    
            
    def evalf(self,freeEnergyArray,points,arraylength,xsteps,phaseNumber,yVal,add=0,**kwargs):
        #print yVal
        from numpy import array
        from numpy import log
        tRange=self.tRange
        prettyE=''
        if self.eqStr!=None and self.tRange!=None and yVal>tRange[0] and yVal<tRange[1]:
            #print 'using old'
            tRange=self.tRange
            equationStr=self.eqStr
            prettyE+=self.prettyE
            
        else:
            tRange=[None,None]
            if self.phase==None or self.phaseElts==None:
                CustomException("Please Select a Phase")
            if len(self.phaseElts)!=2:
                CustomException("Only Works with Binaries Now")
            parameters=self.phases[self.phase]['parameters']
            #print parameters
            #equationStrBase='%s*T*(x%s*log(x%s)+x%s*log(x%s))'%(self.R,self.phaseElts[0],self.phaseElts[0],self.phaseElts[1],self.phaseElts[1])
            equationStr=''
            for p in parameters.keys():
                
                par=parameters[p]
                nI=par['nameInfo']
                #print nI
                comps=nI['components'][0]
                if len(comps)==1 and comps[0] in self.phaseElts:
                    index=0
                    if yVal<float(par['minTemp']):
                        CustomException("Below Minimum Temperature, Cannot do anything")
                    else:
                        if tRange[0]==None or float(par['minTemp'])>tRange[0]:
                            tRange[0]=float(par['minTemp'])
                    #print p,par
                    #print par['maxTemps']
                    if yVal>float(par['maxTemps'][-1]):
                        CustomException("Above Maximum Temperature for data, Cannot do anything")
                    else:
                        found=False
                        
                        while found==False:
                            if float(par['maxTemps'][index])>yVal:
                                found=True
                                if tRange[1]==None or tRange[1]>float(par['maxTemps'][index]):
                                    tRange[1]=float(par['maxTemps'][index])
                                if index>0:
                                    if float(par['maxTemps'][index-1])>tRange[0]:
                                        tRange[0]=float(par['maxTemps'][index-1])
                            else:
                                index+=1
                    
                    #print par['maxTemps']
                    equationStr=equationStr+"+x%s*(%s)"%(comps[0],par['expressions'][index])
                    prettyE+="+x%s*(%s)"%(comps[0],p)
                if len(comps)==2 and comps[0] in self.phaseElts and comps[1] in self.phaseElts:
                    index=0
                    if yVal<float(par['minTemp']):
                        CustomException("Below Minimum Temperature, Cannot do anything")
                    else:
                        if tRange[0]==None or float(par['minTemp'])>tRange[0]:
                            tRange[0]=float(par['minTemp'])
                    if yVal>float(par['maxTemps'][-1]):
                        CustomException("Above Maximum Temperature for data, Cannot do anything")
                    else:
                        found=False
                        
                        while found==False:
                            if float(par['maxTemps'][index])>yVal:
                                found=True
                                if tRange[1]==None or tRange[1]>float(par['maxTemps'][index]):
                                    tRange[1]=float(par['maxTemps'][index])
                                if index>0:
                                    if float(par['maxTemps'][index-1])>tRange[0]:
                                        tRange[0]=float(par['maxTemps'][index-1])
                            else:
                                index+=1
                    
                    equationStr=equationStr+"+x%s*x%s*(x%s-x%s)**%s*(%s)"%(comps[0],comps[1],comps[0],comps[1],nI['power'],par['expressions'][index])
                    prettyE+="+x%s*x%s*(x%s-x%s)**%s*(%s)"%(comps[0],comps[1],comps[0],comps[1],nI['power'],p)
#                    xAg*xCu*(xAg-xCu)^1*L(FCC_A1,AG,CU:VA;1)
#                    xAg*xCu*L(FCC_A1,AG,CU:VA;0)
            equationStr=equationStr.replace("#","")
            prettyE=prettyE.replace("#","")
            #prettyE+=equationStr
            
            tRange,equationStr=self._replaceFunctions(tRange,equationStr,yVal)
            equationStr=equationStr.replace("LN","log")
        self.tRange=tRange
        self.eqStr=equationStr
        equationStrBase='%s*T*(x%s*log(x%s)+x%s*log(x%s))'%(self.R,self.phaseElts[0],self.phaseElts[0],self.phaseElts[1],self.phaseElts[1])
        prettyE=equationStrBase+prettyE
        self.prettyE=prettyE

        #print prettyE
        arrayP=points[(phaseNumber*xsteps+1):((phaseNumber+1)*xsteps)]
        sumstr=equationStrBase+equationStr
        str=sumstr.replace("x%s"%self.phaseElts[1],'arrayP')
        str=str.replace("x%s"%self.phaseElts[0],'(1-arrayP)')
        
            
        T=yVal
#        print arrayP
#        print tRange
#        print equationStr
#        print str
        c=eval(str)
#        print c
        
        if add==1:
            freeEnergyArray[(phaseNumber*xsteps+1):((phaseNumber+1)*xsteps)]+=c
        else:
            freeEnergyArray[(phaseNumber*xsteps+1):((phaseNumber+1)*xsteps)]=c
	#print c
	#print len(c) 
	#print len(freeEnergyArray[(phaseNumber*xsteps+1):((phaseNumber+1)*xsteps)])
	#print freeEnergyArray[((phaseNumber+1)*xsteps)-1]
        ### special case for 0
        sumstr=equationStr
        str=sumstr.replace("x%s"%self.phaseElts[1],'(0)')
        str=str.replace("x%s"%self.phaseElts[0],'(1)')
	#print str
        if add==1:
            freeEnergyArray[phaseNumber*xsteps]+=eval(str)
        else:
            freeEnergyArray[phaseNumber*xsteps]=eval(str)
	sumstr=equationStr
        str=sumstr.replace("x%s"%self.phaseElts[1],'(1)')
        str=str.replace("x%s"%self.phaseElts[0],'(0)')
	#print str
        if add==1:
            freeEnergyArray[((phaseNumber+1)*xsteps)-1]+=eval(str)
        else:
            freeEnergyArray[((phaseNumber+1)*xsteps)-1]=eval(str)
	#print freeEnergyArray[phaseNumber*xsteps+xsteps-1]
#freeEnergyArray[phaseNumber*xsteps+1]+40*(freeEnergyArray[phaseNumber*xsteps+1]-freeEnergyArray[phaseNumber*xsteps+2])
                
            
        
        
        
            
            
        
