import Gnuplot
from gibbs.viewers import Viewer
from gibbs.solvers import SolverCore
from gibbs.tools.gnuplotcompat import GGnuplotData

class GnuplotViewer(Viewer):
    def __init__(self,vars=None,persist=1,type='lines',**kwargs):
        Viewer.__init__(self,vars,**kwargs)
        self.g=Gnuplot.Gnuplot(persist=persist)
        self.g('set style data lines')
        self.g('set style line 1 lt 3 lw 3')
        self.g('set style line 2 lt 1 lw 1')
        self.g('set style line 3 lt 2 lw 1')
        self.setTitle()
        self.g('set term x11')
        
    def setTitle(self,title=None):
        tempT= title or self.getOption('title') or ''
        self.g('set title "' + tempT + '"')
        
    def setLabels(self,xLabel=None,yLabel=None):
        xLabel= xLabel or self.getOption('xlabel') or 'x'
        self.g.xlabel(xLabel)
        yLabel= yLabel or self.getOption('ylabel') or 'y'
        self.g.ylabel(yLabel)
        
    def preparePlots(self,**kwargs):
        self.setOptions(**kwargs)
        self.setLabels()
        self.setTitle()
        self.setLimits()
        
    def setLimits(self):
        limits=self.getOption('limits')
        if limits!=None:
            if type(limits)!=type([]) or (len(limits)!=2 and len(limits)!=4):
                if self.verbose:
                    print 'limits expects list of type: [xmin,xmax,ymin,ymax]\n you gave:',limits
                return
            else:
                self.g('set xrange [' + str(limits[0])  + ':' + str(limits[1]) + ']')
                if len(limits)==4:
                    self.g('set yrange [' + str(limits[2])  + ':' + str(limits[3]) + ']')
                
        
    def getColor(self,line=0):
        colors=self.getOption('colorList')
        default_cl=['r','g','b','m','c','br','o','do']
        if colors==None:
            return line+2
        elif type(colors)==type([]):
            line=line%len(colors)
            color = colors[line%len(colors)]            
            if type(color)==type(0):
                return color
            elif type(color)==type('') and color in default_cl:
                return (default_cl.index(color)+1)
            else:
                return line+2
        else:
            return line+2
                
        
            
    def __call__(self,**kwargs):
        self.plot(**kwargs)
        
    def plot(self,**kwargs):
        self.preparePlots(**kwargs)
        d=[]
        var=self.getOption('vars')
        if var and isinstance(var,SolverCore):
            z=0
            if var.type()=='1Dimensional':
                for i in range(len(var.DG)):
                    a=i*var.steps
                    b=i*var.steps+var.steps
                    d.append(GGnuplotData(var.P[a:b],var.L[a:b],title=str('phase %i'%i),with_=str('lines linecolor %i'%(self.getColor(i)))))
                    #d.append(GGnuplotData(var.P[a:b],var.L[a:b],title=str('phase %i'%i),with_=str('linestyle 1')))


                try:
                    l=len(var.Px)/2
                    for i in range(l):
                        Px=[var.Px[i*2],var.Px[i*2+1]]
                        Py=[var.Py[i*2],var.Py[i*2+1]]
                        d.append(GGnuplotData(Px,Py,with_=str('lines linecolor 1')))                        
                except:
                    pass
                    
            elif var.type()=='2Dimensional': 
                d.append(GGnuplotData(var.Xdata(),var.Ydata(), with_=str('points linecolor 0 pointtype 11')))
                for i in range(len(var.tempList)):
                    d.append(GGnuplotData(var.xtempList[i],var.tempList[i], with_=str('line linecolor 1 linewidth 1')))

                
        else:
            d.append(GGnuplotData(var.P,var.L, with_=str('points linecolor 0 pointtype 3')))
            
        apply(self.g.plot,d)
        #del self.vars
        #del var
        
    def save(self,file,enhanced=1,color=1,**kwargs):
        self.plot(**kwargs) 
       # self.g.hardcopy(file,enhanced,color)
        self.g.hardcopy(file, enhanced=enhanced, color=1,solid=1)
        
