# -*- coding: utf-8 -*-
import pylab as plt
from gibbs.viewers import Viewer
from gibbs.solvers import SolverCore
from gibbs.tools.utils import CustomException


class MatplotlibViewer(Viewer):
    def __init__(self,vars=None,limits=None,title=None,figureNumber=1,aspectRatio='auto',**kwargs):
        Viewer.__init__(self,vars=vars,limits=limits,title=title)
        self.optionsList.extend(['figureNumber','ftitle','aspectRatio'])
        self.fig=None
        self.setOptions(figureNumber=figureNumber,ftitle=title,aspectRatio=aspectRatio,**kwargs)
        
    def preparePlots(self,**kwargs):
        self.setOptions(**kwargs)
        self.setLabels()
        
    def getLimits(self):
        padAmount=0.05
        c=self.getOption('limits')
        var=self.getOption('vars')
        if c!=None and len(c)==4:
            a=c[:]
            if var!=None:
                xRange=max(var.P)-min(var.P)
                yRange=max(var.L)-min(var.L)
                b=[min(var.P)-xRange*padAmount,max(var.P)+xRange*padAmount,min(var.L)-yRange*padAmount,max(var.L)+yRange*padAmount]
                #print 'b',b
                #print 'a',c
            if a[0]==None:
                if var!=None:
                    a[0]=b[0]
                else:
                    a[0]=0.
            if a[1]==None:
                if var!=None:
                    a[1]=b[1]
                else:
                    a[1]=1.
            if a[2]==None:
                if var!=None:
                    a[2]=b[2]
                else:
                    a[2]=0.
            if a[3]==None:
                if var!=None:
                    a[3]=b[3]
                else:
                    a[3]=1.
            return map(float,a)
        else:
            
            if var!=None:
                xRange=max(var.P)-min(var.P)
                yRange=max(var.L)-min(var.L)
                return [min(var.P)-xRange*padAmount,max(var.P)+xRange*padAmount,min(var.L)-yRange*padAmount,max(var.L)+yRange*padAmount]
            else:
                return [0.,1.,0.,1.]
    
    def getXTicks(self,nb_ticks=5):
        from numpy import arange
        if nb_ticks<2:
            nb_ticks=2
        c=self.getOption('limits')
        if c!=None:
            a=c[:]
        else:
            a=None
        if a==None or len(a)!=4:
            var=self.getOption('vars')
            if var==None:
                a=[0,1,0,1]
            else:
                a=[min(var.P),max(var.P),min(var.L),max(var.L)]
        var=self.getOption('vars')
        if a[0]==None:
            if var!=None:
                a[0]=min(var.P)
            else:
                a[0]=0.0
        
        if a[1]==None:
            if var!=None:
                a[1]=max(var.P)
            else:
                a[1]=1.0
            
        xRange=a[1]-a[0]
        c=arange(int(nb_ticks))
        a=a[0]+c*xRange/float((nb_ticks-1))
        return list(a)
        
    def getYTicks(self,nb_ticks=5):
        from numpy import arange
        if nb_ticks<2:
            nb_ticks=2
        c=self.getOption('limits')
        if c!=None:
            a=c[:]
        else:
            a=None
        if a==None or len(a)!=4:
            var=self.getOption('vars')
            if var==None:
                a=[0,1,0,1]
            else:
                a=[min(var.P),max(var.P),min(var.L),max(var.L)]
                
        var=self.getOption('vars')
        if a[2]==None:
            if var!=None:
                a[2]=min(var.L)
            else:
                a[2]=0.0
        
        if a[3]==None:
            if var!=None:
                a[3]=max(var.L)
            else:
                a[3]=1.0
            

        yRange=a[3]-a[2]
        c=arange(int(nb_ticks))
        a=a[2]+c*yRange/(nb_ticks-1)
        return list(a)
        
        

                
                
    def getColor(self,line=0):
        colors=self.getOption('colorList')
        default_cl=['g','b','m','c','brown','orange']
        if colors==None:
            return default_cl[line%len(default_cl)]
        elif type(colors)==type([]):
            line=line%len(colors)
            color = colors[line%len(colors)]            
            if type(color)==type(''):
                return color
            elif type(color)==type(0):
                return default_cl[color]
            else:
                return default_cl[line%len(default_cl)]
        else:
            return default_cl[line%len(default_cl)]
        
    def __call__(self,**kwargs):
        self.plot(**kwargs)
      
    def _getAspect(self):
        aspect=self.getOption('aspectRatio')
        #print aspect
        if type(aspect) not in [type(1.),type(1)]:
            return aspect
        else:
            b=self.getLimits()
            ar=float(b[1]-b[0])/float(b[3]-b[2])*aspect
            return ar
    def plot(self,**kwargs):
        self.setOptions(**kwargs)
        import pylab as plt
        plt.ion()
        fig=plt.figure(self.getOption('figureNumber'))
        plt.ioff()
        supT=self.getOption('ftitle')
        if supT!=None:
            fig.suptitle(supT)
        fig.subplots_adjust(left=0.15,hspace=0.7, wspace=0.5)
        try:
            subplot=kwargs['subplot']
        except:
            subplot='111'
        #print self.getOption('aspectRatio')
        ax=fig.add_subplot(subplot)
        ax.clear()
        ax.set_aspect(self._getAspect(),'box')

        try:
            ax.set_xlabel(kwargs['xlabel'])
        except:
            pass
        try:
            ax.set_ylabel(kwargs['ylabel'])
        except:
            pass
        try:
            ax.set_title(kwargs['title'])
        except:
            pass
        var=self.getOption('vars')

        if var.type() not in ['1Dimensional','2Dimensional']:
            print 'Please use the correct Viewers'
        else:
            if isinstance(var,SolverCore):
                if var.type()=='1Dimensional':
                    for i in range(len(var.DG)):

                        a=i*var.steps
                        b=i*var.steps+var.steps

                        ax.plot(var.P[a:b],var.L[a:b],color=self.getColor(i))

                        try:
                            l=len(var.Px)/2
                            for i in range(l):
                                Px=[var.Px[i*2],var.Px[i*2+1]]
                                Py=[var.Py[i*2],var.Py[i*2+1]]
                                ax.plot(Px,Py,color='r')                        
                        except:
                            pass
                elif var.type()=='2Dimensional':
                    ax.plot(var.Xdata(),var.Ydata(),'.',color='k')
                    for i in range(len(var.tempList)):
                        ax.plot(var.xtempList[i],var.tempList[i],color='r',lw=2)
            ax.set_xlim(self.getLimits()[0:2])
            ax.set_ylim(self.getLimits()[2:4])
            #print self.getLimits()
            xticks=self.getXTicks()
            ax.set_xticks(xticks)
            if kwargs.has_key('xmode') and kwargs['xmode']=='percent':
                xtickL=['%.2f'%(i*100) for i in xticks]
                ax.set_xticklabels(xtickL)
            yticks=self.getYTicks()
            yticklabels=['%.2f'%i for i in yticks]
            ax.set_yticks(yticks)
            ax.set_yticklabels(yticklabels)
            
            self.fig=fig
            if kwargs.has_key('mode') and kwargs['mode']=='show':
                plt.draw()
                plt.show()
            else:
                plt.draw()
            plt.ion()
    def show(self):
        plt.ioff()
        plt.show()
        plt.ion()
    def save(self,file=None,dpi=None,**kwargs):
        path=self.getFilePath(file=file,**kwargs)
        if self.fig==None:
            CustomException('This Viewer does not have a figure yet... Have you plotted a figure?')
        print 'saving at: %s'%path
        #self.plot(**kwargs) 
       # self.g.hardcopy(file,enhanced,color)
       
        self.fig.savefig(path,dpi=dpi)
    
            
            
            
