#! /usr/bin/env python
#
#
#    Created by Thomas Cool on 1-Sept-2009.
#    Copyright 2009 Thomas Cool, R. Edwin Garcia
#    This file is part of gibbs.
#
#    gibbs is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#

import os
from distutils.core import setup, Extension

from distutils.core import Command
base_version='0.5.'
import sys
files = ["gibbs/variables/*"]
modules = [
    'gibbs',
	'gibbs.variables',
	'gibbs.tools',
	'gibbs.solvers',
	'gibbs.solvers.qhsolver',
    'gibbs.solvers.qhsolver.qhbinarysolver',
    'gibbs.solvers.qhsolver.qhTernarySolver',
	'gibbs.variables.regularfreeenergy',
    'gibbs.variables.idealfreeenergy',
    'gibbs.variables.quasiregularfreeenergy',
    'gibbs.variables.ternaryregularfreeenergy',
    'gibbs.variables.pyvariable',
    'gibbs.variables.datafilevariable',
    'gibbs.variables.thermocalcdbvariable',
    'gibbs.viewers',
    'gibbs.viewers.gnuplot',
    'gibbs.viewers.matplotlib',
	]
import numpy

# Obtain the numpy include directory.  This logic works across numpy versions.
try:
    numpy_include = numpy.get_include()
except AttributeError:
    numpy_include = numpy.get_numpy_include()
    
def version():
	try:
		versionFile = open('.svn/entries', 'r')
		end = False
		for line in versionFile:
			if end == True:
				version = base_version + line.split()[0]
				break
			if 'dir' in line:
				end = True
	except:
		version = base_version

	a = open('./gibbs/version.py', 'w')
	b='version = "%s"'%version
	a.write(b)

	return version

class clean(Command):
    """Cleans *.pyc and debian trashs, so you should get the same copy as

    is in the svn.
    """

    description = "Clean everything"
    user_options = [("all","a","the same")]

    def initialize_options(self):
        print 'clean'
        self.all = None

    def finalize_options(self):
        pass

    def run(self):
        print 'clean'
        import os
        os.system("py.cleanup")
        os.system("rm -f python-build-stamp-2.4")
        os.system("rm -f MANIFEST")
        os.system("rm -rf build")
        os.system("rm -rf dist")
StdExtensions=['gibbs/variables/regularfreeenergy/QHRegularFreeEnergyC',
               'gibbs/variables/idealfreeenergy/QHIdealFreeEnergyC',
               'gibbs/variables/quasiregularfreeenergy/QHQuasiRegularFreeEnergyC',
               'gibbs/solvers/qhsolver/QHSolve',
               'gibbs/variables/ternaryregularfreeenergy/QHTernaryRegularFreeEnergyC',
               'gibbs/variables/datafilevariable/QHDataFileVariableMethods'
#               'gibbs/solvers/qhsolver/qhTernarySolver/QHTernarySolve',
#               'gibbs/tools/QHTest',
               ]
Extensions=[]
includeDirs=['/opt/local/include',numpy_include,'includes'];
a=os.getcwd()+'/includes/'
SWINC=['includes']
libraryDirs=['/opt/local/lib'];

for extstr in StdExtensions:
    import os
    b=extstr.split('/')
    cSource=extstr+'.c'
    iSource=extstr+'.i'
    outdir='./build/'+'/'.join(b[:-1])
    c=b[:]
    c[-1]='_'+c[-1]
    newP='.'.join(c)
    print '***\n','Extension: ',extstr,'\n',cSource,iSource,'\n',outdir,'\n',newP,'\n***'
    ext=Extension(newP,sources=[cSource,iSource],swig_opts=['-I%s'%a,'-outdir',outdir],library_dirs=libraryDirs,include_dirs=includeDirs,libraries=['qhull'])
    Extensions.append(ext)
        

#version_nb = version()
reFeExt	= Extension("gibbs.variables.regularfreeenergy._QHRegularFreeEnergyC",
		sources=["gibbs/variables/regularfreeenergy/QHRegularFreeEnergyC.c", "gibbs/variables/regularfreeenergy/QHRegularFreeEnergyC.i"],
		swig_opts=['-I%s'%a,'-outdir','./build/gibbs/variables/regularfreeenergy/'],
        library_dirs=libraryDirs,
        include_dirs=includeDirs,
        libraries=['qhull'])
idFeExt= Extension("gibbs.variables.idealfreeenergy._QHIdealFreeEnergyC",
		sources=["gibbs/variables/idealfreeenergy/QHIdealFreeEnergyC.c", "gibbs/variables/idealfreeenergy/QHIdealFreeEnergyC.i"],
		swig_opts=['-outdir', './build/gibbs/variables/idealfreeenergy/'],
        library_dirs=libraryDirs,
        include_dirs=includeDirs,
        libraries=['qhull'])
qreFeExt = Extension("gibbs.variables.quasiregularfreeenergy._QHQuasiRegularFreeEnergyC",
		sources=["gibbs/variables/quasiregularfreeenergy/QHQuasiRegularFreeEnergyC.c", "gibbs/variables/quasiregularfreeenergy/QHQuasiRegularFreeEnergyC.i"],
		swig_opts=['-I%s'%a,'-outdir','./build/gibbs/variables/quasiregularfreeenergy/'],
        library_dirs=libraryDirs,
        include_dirs=includeDirs,
        libraries=['qhull'])


mainModule= Extension("gibbs/solvers/qhsolver/_QHSolve",
		sources=["gibbs/solvers/qhsolver/QHSolve.c","gibbs/solvers/qhsolver/QHSolve.i"],
		swig_opts=['-outdir', './build/gibbs/solvers/qhsolver/'],
        library_dirs=libraryDirs,
        include_dirs=includeDirs,
        libraries=['qhull'])

setup(
        name    = 'gibbs',
        version = version(),
    	packages= modules,
		author  = 'Thomas Cool',
        ext_modules = Extensions#[qreFeExt,mainModule,reFeExt,idFeExt]
	

)
